@@ -13,18 +13,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 #ifndef ais_merge_h
 #define ais_merge_h
 
@@ -55,33 +55,34 @@
     AISResources to actually open the AIS database and locate AIS resources
     for a particular primary data source. This class  dereferences AIS
     resources returned by AISResources and merges that with an existing DAP
-    object (currently only the DAS object is supported. 02/25/03 jhrg). 
+    object (currently only the DAS object is supported. 02/25/03 jhrg).
 
     @brief Merge AIS resources. */
-class AISMerge {
+class AISMerge
+{
 private:
     AISResources d_ais_db;
-    HTTPConnect d_http;		// used to access remote resources
+    HTTPConnect d_http;  // used to access remote resources
 
     friend class AISMergeTest;
 public:
     /** Initialize an instance so that it reads from \c database.
-	@param database Name of a database of AIS resources.
-	@exception AISDatabaseReadFailed Thrown if the named database cannot
-	be opened. */
-    AISMerge(const string &database) throw (AISDatabaseReadFailed) :
-	d_ais_db(database), d_http(RCReader::instance()) { }
+    @param database Name of a database of AIS resources.
+    @exception AISDatabaseReadFailed Thrown if the named database cannot
+    be opened. */
+    AISMerge(const string &database) throw(AISDatabaseReadFailed) :
+            d_ais_db(database), d_http(RCReader::instance())
+    { }
 
     /** Destroy an instance. This is explicitly declared virtual to support
-	subclassing. */
-    virtual ~AISMerge() {}
+    subclassing. */
+    virtual ~AISMerge()
+    {}
 
     // Change this when HTTPConnect/HTTPCache are changed.
-    virtual Response *get_ais_resource(const string &res) 
-	throw(Error, InternalErr);
+    virtual Response *get_ais_resource(const string &res);
 
-    virtual void merge(const string &primary, DAS &das) 
-	throw(Error, InternalErr);
+    virtual void merge(const string &primary, DAS &das);
 };
 
 #endif // ais_merge_h
