@@ -41,12 +41,10 @@
 #include "debug.h"
 #include "InternalErr.h"
 #include "escaping.h"
-//#include "ArrayIterAdapter.h"
 
 #include <algorithm>
 #include <functional>
 
-
 using namespace std;
 
 void
@@ -82,15 +80,13 @@ Array::update_length(int)
 // Construct an instance of Array. The (BaseType *) is assumed to be
 // allocated using new - The dtor for Vector will delete this object.
 
-/** The Array constructor requires the name of the variable to be
-    created, and the type of data the Array is to hold.  The name
-    may be omitted, which will create a nameless variable.  The
-    template pointer may also be omitted.  Note that if the template
-    pointer is omitted when the Array is created, it <i>must</i> be
-    added (with <tt>add_var()</tt>) before <tt>read()</tt> or
-    <tt>deserialize()</tt> is called.
+/** Build an array with a name and an element type. The name may be omitted,
+    which will create a nameless variable. The template (element type) pointer
+    may also be omitted, but if it is omitted when the Array is created, it 
+    \e must be added (with \c add_var()) before \c read() or \c deserialize()
+    is called.
 
-    @note Force the Array::add_var() method to be used to add \e v.
+    @todo Force the Array::add_var() method to be used to add \e v.
     This version of add_var() calls Vector::add_var().
 
     @param n A string containing the name of the variable to be
@@ -101,7 +97,7 @@ Array::update_length(int)
 */
 Array::Array(const string &n, BaseType *v) : Vector(n, 0, dods_array_c)
 {
-    add_var(v);
+    add_var(v); // Vector::add_var() stores null is v is null
 }
 
 /** @brief The Array copy constructor. */
@@ -198,8 +194,9 @@ Array::append_dim(int size, string name)
     d.stop = size - 1;
     d.stride = 1;
     d.c_size = size;
+#if 0
     d.selected = true;  // assume all dims selected.
-
+#endif
     _shape.push_back(d);
 
     update_length(size);
@@ -221,26 +218,28 @@ Array::reset_constraint()
         (*i).stop = (*i).size - 1;
         (*i).stride = 1;
         (*i).c_size = (*i).size;
-
+#if 0
         (*i).selected = true;
-
+#endif
         update_length((*i).size);
     }
 }
 
 
 /** Tell the Array object to clear the constraint information about
-    dimensions. Do this <i>once</i> before calling
-    <tt>add_constraint()</tt> for each new constraint expression. Only
-    the dimensions explicitly selected using <tt>add_constraint()</tt>
-    will be sent.
+    dimensions. Do this <i>once</i> before calling <tt>add_constraint()</tt> 
+    for each new constraint expression. Only the dimensions explicitly 
+    selected using <tt>add_constraint()</tt> will be sent.
 
+    @deprecated This should never be used. 
     @brief Clears the projection; add each projected dimension explicitly using
     <tt>add_constraint</tt>.
 */
 void
 Array::clear_constraint()
 {
+    reset_constraint();
+#if 0
     for (Dim_iter i = _shape.begin(); i != _shape.end(); i++) {
         (*i).start = 0;
         (*i).stop = 0;
@@ -250,15 +249,14 @@ Array::clear_constraint()
     }
 
     set_length(-1);
+#endif
 }
 
-// the start and stop indexes are inclusive.
-
 // Note: MS VC++ won't tolerate embedded newlines in strings, hence the \n
 // is explicit.
 static const char *array_sss = \
-                               "Invalid constraint parameters: At least one of the start, stride or stop \n\
-                               specified do not match the array variable.";
+"Invalid constraint parameters: At least one of the start, stride or stop \n\
+specified do not match the array variable.";
 
 /** Once a dimension has been created (see append_dim()), it can
     be constrained.  This will make the array appear to the rest
@@ -301,9 +299,9 @@ Array::add_constraint(Dim_iter i, int start, int stride, int stop)
     d.c_size = (stop - start) / stride + 1;
 
     DBG(cerr << "add_constraint: c_size = " << d.c_size << endl);
-
+#if 0
     d.selected = true;
-
+#endif
     update_length(d.c_size);
 }
 
@@ -331,10 +329,12 @@ Array::dim_end()
 */
 
 unsigned int
-Array::dimensions(bool constrained)
+Array::dimensions(bool /*constrained*/)
 {
     unsigned int dim = 0;
     for (Dim_citer i = _shape.begin(); i != _shape.end(); i++) {
+        dim++;
+#if 0
         if (constrained) {
             if ((*i).selected)
                 dim++;
@@ -342,6 +342,7 @@ Array::dimensions(bool constrained)
         else {
             dim++;
         }
+#endif
     }
 
     return dim;
@@ -369,7 +370,14 @@ Array::dimension_size(Dim_iter i, bool constrained)
 {
     int size = 0;
 
-    if (!_shape.empty())
+    if (!_shape.empty()) {
+        if (constrained)
+            size = (*i).c_size;
+        else
+            size = (*i).size;
+    }
+    
+#if 0    
         if (constrained) {
             if ((*i).selected)
                 size = (*i).c_size;
@@ -378,6 +386,7 @@ Array::dimension_size(Dim_iter i, bool constrained)
         }
         else
             size = (*i).size;
+#endif
 
     return size;
 }
@@ -401,11 +410,17 @@ Array::dimension_size(Dim_iter i, bool constrained)
     @return The desired start index.
 */
 int
-Array::dimension_start(Dim_iter i, bool constrained)
+Array::dimension_start(Dim_iter i, bool /*constrained*/)
 {
+    return (!_shape.empty()) ? (*i).start : 0;
+    
+#if 0
     int start = 0;
 
     if (!_shape.empty())
+        start = (*i).start;
+        
+#if array_selected
         if (constrained) {
             if ((*i).selected)
                 start = (*i).start;
@@ -414,8 +429,10 @@ Array::dimension_start(Dim_iter i, bool constrained)
         }
         else
             start = (*i).start;
+#endif
 
     return start;
+#endif
 }
 
 /** Use this function to return the stop index of an array
@@ -437,11 +454,17 @@ Array::dimension_start(Dim_iter i, bool constrained)
     @return The desired stop index.
 */
 int
-Array::dimension_stop(Dim_iter i, bool constrained)
+Array::dimension_stop(Dim_iter i, bool /*constrained*/)
 {
+    return (!_shape.empty()) ? (*i).stop : 0;
+    
+#if 0
     int stop = 0;
 
     if (!_shape.empty())
+        stop = (*i).stop;
+    
+#if array_selected
         if (constrained) {
             if ((*i).selected)
                 stop = (*i).stop;
@@ -450,8 +473,10 @@ Array::dimension_stop(Dim_iter i, bool constrained)
         }
         else
             stop = (*i).stop;
+#endif
 
     return stop;
+#endif
 }
 
 /** Use this function to return the stride value of an array
@@ -474,11 +499,17 @@ Array::dimension_stop(Dim_iter i, bool constrained)
     is TRUE and the dimension is not selected.
 */
 int
-Array::dimension_stride(Dim_iter i, bool constrained)
+Array::dimension_stride(Dim_iter i, bool /*constrained*/)
 {
+    return (!_shape.empty()) ? (*i).stride : 0;
+    
+#if 0
     int stride = 0;
 
     if (!_shape.empty())
+        stride = (*i).stride;
+    
+#if array_selected
         if (constrained) {
             if ((*i).selected)
                 stride = (*i).stride;
@@ -487,8 +518,10 @@ Array::dimension_stride(Dim_iter i, bool constrained)
         }
         else
             stride = (*i).stride;
+#endif
 
     return stride;
+#endif
 }
 
 /** This function returns the name of the dimension indicated with
@@ -543,8 +576,10 @@ Array::print_decl(FILE *out, string space, bool print_semi,
     var()->print_decl(out, space, false, constraint_info, constrained);
 
     for (Dim_citer i = _shape.begin(); i != _shape.end(); i++) {
+#if 0
         if (constrained && !((*i).selected))
             continue;
+#endif
         fprintf(out, "[") ;
         if ((*i).name != "") {
             fprintf(out, "%s = ", id2www((*i).name).c_str()) ;
@@ -562,18 +597,79 @@ Array::print_decl(FILE *out, string space, bool print_semi,
     }
 }
 
+/** Prints a declaration for the Array.  This is what appears in a
+    DDS.  If the Array is constrained, the declaration will reflect
+    the size of the Array once the constraint is applied.
+
+    @brief Prints a DDS entry for the Array.
+
+    @param out Write the output to this ostream.
+    @param space A string containing spaces to precede the
+    declaration.
+    @param print_semi A boolean indicating whether to print a
+    semi-colon after the declaration.  (TRUE means ``print a
+    semi-colon.'')
+    @param constraint_info A boolean value.  See
+    <tt>BaseType::print_decl()</tt>.
+    @param constrained This argument should be TRUE if the Array is
+    constrained, and FALSE otherwise.
+*/
+void
+Array::print_decl(ostream &out, string space, bool print_semi,
+                  bool constraint_info, bool constrained)
+{
+    if (constrained && !send_p())
+        return;
+
+    // print it, but w/o semicolon
+    var()->print_decl(out, space, false, constraint_info, constrained);
+
+    for (Dim_citer i = _shape.begin(); i != _shape.end(); i++) {
+#if 0
+        if (constrained && !((*i).selected))
+            continue;
+#endif
+	out << "[" ;
+        if ((*i).name != "") {
+	    out << id2www((*i).name) << " = " ;
+        }
+        if (constrained) {
+	    out << (*i).c_size << "]" ;
+        }
+        else {
+	    out << (*i).size << "]" ;
+        }
+    }
+
+    if (print_semi) {
+	out << ";\n" ;
+    }
+}
+
 void
 Array::print_xml(FILE *out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Array");
 }
 
+void
+Array::print_xml(ostream &out, string space, bool constrained)
+{
+    print_xml_core(out, space, constrained, "Array");
+}
+
 void
 Array::print_as_map_xml(FILE *out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Map");
 }
 
+void
+Array::print_as_map_xml(ostream &out, string space, bool constrained)
+{
+    print_xml_core(out, space, constrained, "Map");
+}
+
 class PrintArrayDim : public unary_function<Array::dimension&, void>
 {
     FILE *d_out;
@@ -621,6 +717,52 @@ Array::print_xml_core(FILE *out, string space, bool constrained, string tag)
     fprintf(out, "%s</%s>\n", space.c_str(), tag.c_str());
 }
 
+class PrintArrayDimStrm : public unary_function<Array::dimension&, void>
+{
+    ostream &d_out;
+    string d_space;
+    bool d_constrained;
+public:
+    PrintArrayDimStrm(ostream &o, string s, bool c)
+            : d_out(o), d_space(s), d_constrained(c)
+    {}
+
+    void operator()(Array::dimension &d)
+    {
+        int size = d_constrained ? d.c_size : d.size;
+        if (d.name.empty())
+	    d_out << d_space << "<dimension size=\"" << size << "\"/>\n" ;
+        else
+	    d_out << d_space << "<dimension name=\"" << id2xml(d.name)
+	          << "\" size=\"" << size << "\"/>\n" ;
+    }
+};
+
+void
+Array::print_xml_core(ostream &out, string space, bool constrained, string tag)
+{
+    if (constrained && !send_p())
+        return;
+
+    out << space << "<" << tag ;
+    if (!name().empty())
+	out << " name=\"" << id2xml(name()) << "\"" ;
+    out << ">\n" ;
+
+    get_attr_table().print_xml(out, space + "    ", constrained);
+
+    BaseType *btp = var();
+    string tmp_name = btp->name();
+    btp->set_name("");
+    btp->print_xml(out, space + "    ", constrained);
+    btp->set_name(tmp_name);
+
+    for_each(dim_begin(), dim_end(),
+             PrintArrayDimStrm(out, space + "    ", constrained));
+
+    out << space << "</" << tag << ">\n" ;
+}
+
 /** Prints the values in ASCII of the entire (constrained) array. This method
     Attempts to make an aesthetically pleasing display. However, it is
     primarily intended for debugging purposes.
@@ -665,6 +807,50 @@ Array::print_array(FILE *out, unsigned int index, unsigned int dims,
     }
 }
 
+/** Prints the values in ASCII of the entire (constrained) array. This method
+    Attempts to make an aesthetically pleasing display. However, it is
+    primarily intended for debugging purposes.
+
+    @param out Write the output to this ostream
+    @param space The space to use in printing.
+    @param print_decl_p A boolean value indicating whether you want
+    the Array declaration to precede the Array value.
+    @brief Print the value given the current constraint.
+*/
+unsigned int
+Array::print_array(ostream &out, unsigned int index, unsigned int dims,
+                   unsigned int shape[])
+{
+    if (dims == 1) {
+	out << "{" ;
+        for (unsigned i = 0; i < shape[0] - 1; ++i) {
+            var(index++)->print_val(out, "", false);
+	    out << ", " ;
+        }
+        var(index++)->print_val(out, "", false);
+	out << "}" ;
+
+        return index;
+    }
+    else {
+	out << "{" ;
+        // Fixed an off-by-one error in the following loop. Since the array
+        // length is shape[dims-1]-1 *and* since we want one less dimension
+        // than that, the correct limit on this loop is shape[dims-2]-1. From
+        // Todd Karakasian.
+        // The saga continues; the loop test should be `i < shape[0]-1'. jhrg
+        // 9/12/96.
+        for (unsigned i = 0; i < shape[0] - 1; ++i) {
+            index = print_array(out, index, dims - 1, shape + 1);
+	    out << "," ;
+        }
+        index = print_array(out, index, dims - 1, shape + 1);
+	out << "}" ;
+
+        return index;
+    }
+}
+
 void
 Array::print_val(FILE *out, string space, bool print_decl_p)
 {
@@ -679,13 +865,13 @@ Array::print_val(FILE *out, string space, bool print_decl_p)
         fprintf(out, " = ") ;
     }
 
-    unsigned int dims = dimensions(true);
-    unsigned int *shape = new unsigned int[dims];
+    unsigned int *shape = new unsigned int[_shape.size()];
     unsigned int index = 0;
-    for (Dim_iter i = _shape.begin(); i != _shape.end(); i++)
+    for (Dim_iter i = _shape.begin(); i != _shape.end() && index < _shape.size(); i++)
         shape[index++] = dimension_size(i, true);
 
-    print_array(out, 0, dims, shape);
+    print_array(out, 0, _shape.size(), shape);
+    
     delete [] shape; shape = 0;
 
     if (print_decl_p) {
@@ -693,6 +879,34 @@ Array::print_val(FILE *out, string space, bool print_decl_p)
     }
 }
 
+void
+Array::print_val(ostream &out, string space, bool print_decl_p)
+{
+    // print the declaration if print decl is true.
+    // for each dimension,
+    //   for each element,
+    //     print the array given its shape, number of dimensions.
+    // Add the `;'
+
+    if (print_decl_p) {
+        print_decl(out, space, false, false, false);
+	    out << " = " ;
+    }
+
+    unsigned int *shape = new unsigned int[dimensions(true)];
+    unsigned int index = 0;
+    for (Dim_iter i = _shape.begin(); i != _shape.end() && index < dimensions(true); ++i)
+        shape[index++] = dimension_size(i, true);
+
+    print_array(out, 0, dimensions(true), shape);
+    
+    delete [] shape; shape = 0;
+
+    if (print_decl_p) {
+	    out << ";\n" ;
+    }
+}
+
 /** This function checks semantic features of the Array.  Currently,
     the only check specific to the Array is that there must be
     dimensions.  The rest is inherited from
@@ -734,7 +948,8 @@ Array::dump(ostream &strm) const
     Dim_citer ie = _shape.end() ;
     unsigned int dim_num = 0 ;
     for (; i != ie; i++) {
-        strm << DapIndent::LMarg << "dimension " << dim_num++ << ":" << endl ;
+        strm << DapIndent::LMarg << "dimension " << dim_num++ << ":"
+	     << endl ;
         DapIndent::Indent() ;
         strm << DapIndent::LMarg << "name: " << (*i).name << endl ;
         strm << DapIndent::LMarg << "size: " << (*i).size << endl ;
@@ -742,8 +957,10 @@ Array::dump(ostream &strm) const
         strm << DapIndent::LMarg << "stop: " << (*i).stop << endl ;
         strm << DapIndent::LMarg << "stride: " << (*i).stride << endl ;
         strm << DapIndent::LMarg << "constrained size: " << (*i).c_size
-        << endl ;
+             << endl ;
+#if 0
         strm << DapIndent::LMarg << "selected: " << (*i).selected << endl ;
+#endif
         DapIndent::UnIndent() ;
     }
     DapIndent::UnIndent() ;
