@@ -37,9 +37,7 @@
 #ifndef _array_h
 #define _array_h 1
 
-
 #include <string>
-
 #include <vector>
 
 #ifndef _dods_limits_h
@@ -99,9 +97,9 @@ class Array: public Vector
 {
 public:
     /** Information about a dimension. Each Array has one or more dimensions.
-     For each of an Array's dimensions, a corresponding instance of this
-     struct holds the natural size, name, constraint information and
-     constrained size.
+        For each of an Array's dimensions, a corresponding instance of this
+        struct holds the natural size, name, constraint information and
+        constrained size.
 
         @note Instead of using this struct's fields directly, use Array's
         dimension accessor methods.
@@ -116,33 +114,38 @@ public:
         int stop;  ///< The constraint end index
         int stride;  ///< The constraint stride
         int c_size;  ///< Size of dimension once constrained
+#if array_selected
         bool selected; ///< \c True if a constraint has been applied to this dimension.
+#endif
     };
 
 private:
     std::vector<dimension> _shape; // list of dimensions (i.e., the shape)
     unsigned int print_array(FILE *out, unsigned int index,
                              unsigned int dims, unsigned int shape[]);
+    unsigned int print_array(ostream &out, unsigned int index,
+                             unsigned int dims, unsigned int shape[]);
 
     friend class ArrayTest;
 
 protected:
     void _duplicate(const Array &a);
     void print_xml_core(FILE *out, string space, bool constrained, string tag);
+    void print_xml_core(ostream &out, string space, bool constrained, string tag);
 
 public:
     /** A constant iterator used to access the various dimensions of an
-    Array. 
+        Array. 
 
         @see dim_begin()
-    @see dim_end() */
+        @see dim_end() */
     typedef std::vector<dimension>::const_iterator Dim_citer ;
     /** An iterator used to access the various dimensions of an
-    Array. Most of the methods that access various properties of a
-    dimension use an instance of Dim_iter.
+        Array. Most of the methods that access various properties of a
+        dimension use an instance of Dim_iter.
 
         @see dim_begin()
-    @see dim_end() */
+        @see dim_end() */
     typedef std::vector<dimension>::iterator Dim_iter ;
 
     Array(const string &n = "", BaseType *v = 0);
@@ -178,15 +181,25 @@ public:
                             bool print_semi = true,
                             bool constraint_info = false,
                             bool constrained = false);
+    virtual void print_decl(ostream &out, string space = "    ",
+                            bool print_semi = true,
+                            bool constraint_info = false,
+                            bool constrained = false);
 
     virtual void print_xml(FILE *out, string space = "    ",
                            bool constrained = false);
+    virtual void print_xml(ostream &out, string space = "    ",
+                           bool constrained = false);
 
     virtual void print_as_map_xml(FILE *out, string space = "    ",
                                   bool constrained = false);
+    virtual void print_as_map_xml(ostream &out, string space = "    ",
+                                  bool constrained = false);
 
     virtual void print_val(FILE *out, string space = "",
                            bool print_decl_p = true);
+    virtual void print_val(ostream &out, string space = "",
+                           bool print_decl_p = true);
 
     virtual bool check_semantics(string &msg, bool all = false);
 
