@@ -43,17 +43,6 @@
 #define _basetype_h 1
 
 
-#ifdef WIN32
-#include <rpc.h>
-#include <winsock2.h>
-#include <xdr.h>
-#define xdr_proc_t int *
-#else
-#include <rpc/types.h>
-#include <netinet/in.h>
-#include <rpc/xdr.h>
-#endif
-
 #include <vector>
 #include <stack>
 #include <iostream>
@@ -75,6 +64,9 @@
 #include "DapObj.h"
 #endif
 
+#include "Marshaller.h"
+#include "UnMarshaller.h"
+
 using namespace std;
 
 class BaseType;   // Forward declarations
@@ -199,11 +191,6 @@ private:
     string _name;  // name of the instance
     Type _type;   // instance's type
 
-    // xdr_coder is used as an argument to xdr procedures that encode groups
-    // of things (e.g., xdr_array()). Each leaf class's constructor must set
-    // this.
-    xdrproc_t _xdr_coder;
-
     bool _read_p;  // true if the value has been read
     bool _send_p;  // Is the variable in the projection?
     bool d_in_selection; // Is the variable in the selection?
@@ -221,8 +208,7 @@ protected:
     void _duplicate(const BaseType &bt);
 
 public:
-    BaseType(const string &n = "", const Type &t = dods_null_c,
-             xdrproc_t xdr = NULL);
+    BaseType(const string &n = "", const Type &t = dods_null_c);
 
     BaseType(const BaseType &copy_from);
     virtual ~BaseType();
@@ -269,8 +255,6 @@ public:
     virtual bool is_in_selection();
     virtual void set_in_selection(bool state);
 
-    xdrproc_t xdr_coder();
-
     virtual void set_parent(BaseType *parent);
     virtual BaseType *get_parent();
 
@@ -325,9 +309,17 @@ public:
                             bool constraint_info = false,
                             bool constrained = false);
 
+    virtual void print_decl(ostream &out, string space = "    ",
+                            bool print_semi = true,
+                            bool constraint_info = false,
+                            bool constrained = false);
+
     virtual void print_xml(FILE *out, string space = "    ",
                            bool constrained = false);
 
+    virtual void print_xml(ostream &out, string space = "    ",
+                           bool constrained = false);
+
     /** @name Abstract Methods */
     //@{
 
@@ -352,6 +344,8 @@ public:
 	caller is responsible for deallocating that memory. Array and
 	values for simple types are stored as C would store an array.
 
+        @deprecated Use value() in the leaf classes.
+         
 	@brief Reads the class data. 
 
 	@param val A pointer to a pointer to the memory into which the
@@ -374,6 +368,8 @@ public:
 	put data into more complex constructor types, use the
 	functions provided by that class.
 
+        @deprecated Use set_value() in the leaf classes.
+         
 	@brief Loads class data.
 
 	@param val A pointer to the data to be inserted into the class
@@ -393,6 +389,22 @@ public:
 	@see Vector::val2buf */
     virtual unsigned int val2buf(void *val, bool reuse = false) = 0;
 
+    /** Similar to using serialize() and deserialize() together in one object. 
+        Data are read as for serialize and those values are stored in the 
+        objects as deserialize() does but does not write and then read data
+        to/from a stream.
+        
+        This method is defined by the various data type classes. It calls the 
+        read() abstract method.
+        
+        @param dataset The (local) name of dataset to be read.
+        @param eval Use this as the constraint expression evaluator.
+        @param dds The Data Descriptor Structure object corresponding
+        to this dataset. See <i>The DODS User Manual</i> for
+        information about this structure. */
+    virtual void intern_data(const string &dataset, ConstraintEvaluator &eval,
+                             DDS &dds);
+
     /** Sends the data from the indicated (local) dataset through the
 	connection identified by the <i>sink</i> parameter. If the
 	data is not already incorporated into the DDS object, read the
@@ -411,9 +423,7 @@ public:
 	@param dds The Data Descriptor Structure object corresponding
 	to this dataset. See <i>The DODS User Manual</i> for
 	information about this structure.
-	@param sink A valid XDR pointer generally created with a call
-	to <tt>new_xdrstdio()</tt>. This typically routes data to a
-	TCP/IP socket.
+	@param m A marshaller used to serialize data types
 	@param ce_eval A boolean value indicating whether to evaluate
 	the DODS constraint expression that may accompany this
 	dataset. The constraint expression is stored in <i>dds</i>.
@@ -424,7 +434,7 @@ public:
 	@exception Error.
 	@see DDS */
     virtual bool serialize(const string &dataset, ConstraintEvaluator &eval,
-                           DDS &dds, XDR *sink, bool ce_eval = true) = 0;
+                           DDS &dds, Marshaller &m, bool ce_eval = true) = 0;
 
     /** Receives data from the network connection identified by the
 	<tt>source</tt> parameter. The data is put into the class data
@@ -435,9 +445,7 @@ public:
 
 	@brief Receive data from the net.
 
-	@param source A valid XDR pointer to the process connection to
-	the net. This is generally created with a call to
-	<tt>new_xdrstdio()</tt>.
+	@param um An UnMarshaller that knows how to deserialize data types
 	@param dds The Data Descriptor Structure object corresponding
 	to this dataset. See <i>The DODS User Manual</i> for
 	information about this structure. This would have been
@@ -449,10 +457,10 @@ public:
 	FALSE, new storage is allocated. If the internal buffer has
 	not been allocated at all, this argument has no effect.
 	@return Always returns TRUE.
-	@exception Error when a problem reading from the XDR stream is
+	@exception Error when a problem reading from the UnMarshaller is
 	found.
 	@see DDS */
-    virtual bool deserialize(XDR *source, DDS *dds, bool reuse = false) = 0;
+    virtual bool deserialize(UnMarshaller &um, DDS *dds, bool reuse = false) = 0;
 
     /** Prints the value of the variable, with its declaration. This
 	function is primarily intended for debugging DODS
@@ -470,6 +478,23 @@ public:
 	variable declaration is printed as well as the value. */
     virtual void print_val(FILE *out, string space = "",
                            bool print_decl_p = true) = 0;
+
+    /** Prints the value of the variable, with its declaration. This
+	function is primarily intended for debugging DODS
+	applications. However, it can be overloaded and used to do
+	some useful things. Take a look at the asciival and writeval
+	clients, both of which overload this to output the values of
+	variables in different ways.
+
+	@brief Prints the value of the variable.
+
+	@param out The output ostream on which to print the value.
+	@param space This value is passed to the print_decl()
+	function, and controls the leading spaces of the output.
+	@param print_decl_p A boolean value controlling whether the
+	variable declaration is printed as well as the value. */
+    virtual void print_val(ostream &out, string space = "",
+                           bool print_decl_p = true) = 0;
     //@}
 };
 
