@@ -87,8 +87,8 @@ public:
     virtual BaseType *ptr_duplicate();
 
     bool serialize(const string &dataset, ConstraintEvaluator &eval,
-                   DDS &dds, XDR *sink, bool ce_eval);
-    bool deserialize(XDR *source, DDS *, bool);
+                   DDS &dds, Marshaller &m, bool ce_eval);
+    bool deserialize(UnMarshaller &um, DDS *, bool);
 
     virtual unsigned int val2buf(void *val, bool reuse = false);
     virtual unsigned int buf2val(void **val);
@@ -98,6 +98,8 @@ public:
 
     virtual void print_val(FILE *out, string space = "",
                            bool print_decl_p = true);
+    virtual void print_val(ostream &out, string space = "",
+                           bool print_decl_p = true);
 
     virtual bool ops(BaseType *b, int op, const string &dataset);
 
