@@ -39,7 +39,7 @@
 //#define DODS_DEBUG
 
 static char rcsid[] not_used =
-    { "$Id: Connect.cc 16612 2007-06-04 22:08:57Z jimg $"
+    { "$Id: Connect.cc 17002 2007-08-27 19:16:51Z pwest $"
     };
 
 #include <stdio.h>
@@ -48,6 +48,7 @@ static char rcsid[] not_used =
 #endif
 
 #include <fstream>
+#include <algorithm>
 
 #include "debug.h"
 #include "DataDDS.h"
@@ -55,11 +56,13 @@ static char rcsid[] not_used =
 #include "escaping.h"
 #include "RCReader.h"
 #include "DDXParser.h"
+#include "XDRFileUnMarshaller.h"
 
 using std::cerr;
 using std::endl;
 using std::ifstream;
 using std::ofstream;
+using std::min;
 
 extern ObjectType get_type(const string &value);
 
@@ -97,21 +100,19 @@ Connect::process_data(DataDDS &data, Response *rs)
     default: {
             // Parse the DDS; throw an exception on error.
             data.parse(rs->get_stream());
-            XDR *xdr_stream = new_xdrstdio(rs->get_stream(), XDR_DECODE);
+	    XDRFileUnMarshaller um( rs->get_stream() ) ;
 
             // Load the DDS with data.
             try {
                 for (DDS::Vars_iter i = data.var_begin(); i != data.var_end();
                      i++) {
-                    (*i)->deserialize(xdr_stream, &data);
+                    (*i)->deserialize(um, &data);
                 }
             }
             catch (Error &e) {
-                delete_xdrstdio(xdr_stream);
                 throw e;
             }
 
-            delete_xdrstdio(xdr_stream);
             return;
         }
 
@@ -179,6 +180,7 @@ Connect::parse_mime(/*FILE *data_source, */Response *rs)
     fgets(line, 255, data_source);
 
     int slen = strlen(line);
+    slen = min(slen, 256); // use min to limit slen to 256 (fortify)
     line[slen - 1] = '\0'; // remove the newline
     if (line[slen - 2] == '\r') // ...and the preceding carriage return
         line[slen - 2] = '\0';
@@ -215,6 +217,7 @@ Connect::parse_mime(/*FILE *data_source, */Response *rs)
 
         fgets(line, 255, data_source);
         slen = strlen(line);
+        slen = min(slen, 256); // use min to limit slen to 256
         line[slen - 1] = '\0';
         if (line[slen - 2] == '\r')
             line[slen - 2] = '\0';
