@@ -25,7 +25,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: ConstraintEvaluator.cc 16612 2007-06-04 22:08:57Z jimg $"
+    {"$Id: ConstraintEvaluator.cc 16936 2007-08-13 20:01:09Z jimg $"
     };
 
 #include "ConstraintEvaluator.h"
@@ -153,7 +153,7 @@ ConstraintEvaluator::append_clause(btp_func func, rvalue_list *args)
     these constants are never accessed from the list. The list is simply
     a convenient way to make sure the constants are disposed of properly.
 
-    the constraint expression parser. */
+    @todo Update this comment (ConstraintEvaluator, not DDS. */
 void
 ConstraintEvaluator::append_constant(BaseType *btp)
 {
