@@ -186,7 +186,9 @@ Constructor::find_matching_container(AttrTable::entry *source,
     operate on the AttrTable instances in a DDS/BaseType instead of those in
     a DAS.
 
-    @param at Get attribute information from this Attribute table. */
+    @param entry Get attribute information from this Attribute table. Note
+    that even though the type of the argument is an AttrTable::entry, the
+    entry \e must be an attribute container.*/
 void
 Constructor::transfer_attributes(AttrTable::entry * entry)
 {
@@ -294,6 +296,31 @@ Constructor::print_decl(FILE *out, string space, bool print_semi,
         fprintf(out, ";\n") ;
 }
 
+void
+Constructor::print_decl(ostream &out, string space, bool print_semi,
+                        bool constraint_info, bool constrained)
+{
+    if (constrained && !send_p())
+        return;
+
+    out << space << type_name() << " {\n" ;
+    for (Vars_citer i = _vars.begin(); i != _vars.end(); i++) {
+        (*i)->print_decl(out, space + "    ", true,
+                         constraint_info, constrained);
+    }
+    out << space << "} " << id2www(name()) ;
+
+    if (constraint_info) { // Used by test drivers only.
+        if (send_p())
+            out << ": Send True";
+        else
+            out << ": Send False";
+    }
+
+    if (print_semi)
+	out << ";\n" ;
+}
+
 class PrintField : public unary_function<BaseType *, void>
 {
     FILE *d_out;
@@ -338,6 +365,50 @@ Constructor::print_xml(FILE *out, string space, bool constrained)
     }
 }
 
+class PrintFieldStrm : public unary_function<BaseType *, void>
+{
+    ostream &d_out;
+    string d_space;
+    bool d_constrained;
+public:
+    PrintFieldStrm(ostream &o, string s, bool c)
+            : d_out(o), d_space(s), d_constrained(c)
+    {}
+
+    void operator()(BaseType *btp)
+    {
+        btp->print_xml(d_out, d_space, d_constrained);
+    }
+};
+
+void
+Constructor::print_xml(ostream &out, string space, bool constrained)
+{
+    if (constrained && !send_p())
+        return;
+
+    bool has_attributes = false; // *** fix me
+    bool has_variables = (var_begin() != var_end());
+
+    out << space << "<" << type_name() ;
+    if (!name().empty())
+	out << " name=\"" << id2xml(name()) << "\"" ;
+
+    if (has_attributes || has_variables) {
+	out << ">\n" ;
+
+        get_attr_table().print_xml(out, space + "    ", constrained);
+
+        for_each(var_begin(), var_end(),
+                 PrintFieldStrm(out, space + "    ", constrained));
+
+	out << space << "</" << type_name() << ">\n" ;
+    }
+    else {
+	out << "/>\n" ;
+    }
+}
+
 /** True if the instance can be flattened and printed as a single table
     of values. For Arrays and Grids this is always false. For Structures
     and Sequences the conditions are more complex. The implementation
