@@ -37,7 +37,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: DAS.cc 16088 2007-03-28 21:42:19Z jimg $"
+    {"$Id: DAS.cc 17002 2007-08-27 19:16:51Z pwest $"
     };
 
 
@@ -182,7 +182,7 @@ DAS::parse(string fname)
     FILE *in = fopen(fname.c_str(), "r");
 
     if (!in) {
-        throw Error(can_not_read_file, "Could not open: " + fname);
+        throw Error(cannot_read_file, "Could not open: " + fname);
     }
 
     parse(in);
@@ -280,6 +280,28 @@ DAS::print(FILE *out, bool dereference)
     fprintf(out, "}\n") ;
 }
 
+/** Creates an ASCII representation of a DAS on the given output
+    stream.
+
+    When an identifier contains a character that contains
+    characters that cannot be present in a URL (e.g., a space)
+    AttrTable::print replaces those characters with WWW
+    escape codes. 7/13/2001 jhrg
+
+    @param out output ostream on which to print the DAS
+    @param dereference If true, follow aliases. Default is false.
+*/
+
+void
+DAS::print(ostream &out, bool dereference)
+{
+    out << "Attributes {\n" ;
+
+    AttrTable::print(out, "    ", dereference);
+
+    out << "}\n" ;
+}
+
 /** @brief dumps information about this object
  *
  * Displays the pointer value of this instance and then calls parent dump
@@ -291,7 +313,7 @@ void
 DAS::dump(ostream &strm) const
 {
     strm << DapIndent::LMarg << "DAS::dump - ("
-    << (void *)this << ")" << endl ;
+         << (void *)this << ")" << endl ;
     DapIndent::Indent() ;
     AttrTable::dump(strm) ;
     DapIndent::UnIndent() ;
