@@ -34,7 +34,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: DDS.cc 16088 2007-03-28 21:42:19Z jimg $"
+    {"$Id: DDS.cc 17002 2007-08-27 19:16:51Z pwest $"
     };
 
 #include <stdio.h>
@@ -600,7 +600,7 @@ DDS::parse(string fname)
     FILE *in = fopen(fname.c_str(), "r");
 
     if (!in) {
-        throw Error(can_not_read_file, "Could not open: " + fname);
+        throw Error(cannot_read_file, "Could not open: " + fname);
     }
 
     try {
@@ -685,6 +685,21 @@ DDS::print(FILE *out)
     return ;
 }
 
+/** @brief Print the entire DDS to the specified ostream. */
+void
+DDS::print(ostream &out)
+{
+    out << "Dataset {\n" ;
+
+    for (Vars_citer i = vars.begin(); i != vars.end(); i++) {
+        (*i)->print_decl(out) ;
+    }
+
+    out << "} " << id2www(name) << ";\n" ;
+
+    return ;
+}
+
 /** @brief Print a constrained DDS to the specified file.
 
     Print those parts (variables) of the DDS structure to OS that
@@ -712,6 +727,33 @@ DDS::print_constrained(FILE *out)
     return;
 }
 
+/** @brief Print a constrained DDS to the specified ostream.
+
+    Print those parts (variables) of the DDS structure to OS that
+    are marked to be sent after evaluating the constraint
+    expression.
+
+    \note This function only works for scalars at the top level.
+
+    @returns true.
+*/
+void
+DDS::print_constrained(ostream &out)
+{
+    out << "Dataset {\n" ;
+
+    for (Vars_citer i = vars.begin(); i != vars.end(); i++) {
+        // for each variable, indent with four spaces, print a trailing
+        // semi-colon, do not print debugging information, print only
+        // variables in the current projection.
+        (*i)->print_decl(out, "    ", true, false, true) ;
+    }
+
+    out << "} " << id2www(name) << ";\n" ;
+
+    return;
+}
+
 class VariablePrintXML : public unary_function<BaseType *, void>
 {
     FILE *d_out;
@@ -761,6 +803,55 @@ DDS::print_xml(FILE *out, bool constrained, const string &)
     fprintf(out, "</Dataset>\n");
 }
 
+class VariablePrintXMLStrm : public unary_function<BaseType *, void>
+{
+    ostream &d_out;
+    bool d_constrained;
+public:
+    VariablePrintXMLStrm(ostream &out, bool constrained)
+            : d_out(out), d_constrained(constrained)
+    {}
+    void operator()(BaseType *bt)
+    {
+        bt->print_xml(d_out, "    ", d_constrained);
+    }
+};
+
+/** Print an XML represnetation of this DDS. This method is used to generate
+    the part of the DDX response. The \c Dataset tag is \e not written by
+    this code. The caller of this method must handle writing that and
+    including the \c dataBLOB tag.
+
+    @param out Destination ostream.
+    @param constrained True if the output should be limited to just those
+    variables that are in the projection of the current constraint
+    expression.
+    @param blob The dataBLOB href. */
+void
+DDS::print_xml(ostream &out, bool constrained, const string &)
+{
+    out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ;
+
+    out << "<Dataset name=\"" << id2xml(name) << "\"\n" ;
+
+    out << "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" ;
+    out << "xmlns=\"" << dods_namespace << "\"\n" ;
+    out << "xsi:schemaLocation=\"" << dods_namespace
+        << "  " << default_schema_location << "\">\n\n" ;
+
+    d_attr.print_xml(out, "    ", constrained);
+
+    out << "\n" ;
+
+    for_each(var_begin(), var_end(), VariablePrintXMLStrm(out, constrained));
+
+    out << "\n" ;
+
+    out << "    <dataBLOB href=\"\"/>\n" ;
+
+    out << "</Dataset>\n" ;
+}
+
 // Used by DDS::send() when returning data from a function call.
 /** @brief Check the semantics of each of the variables represented in the
     DDS.
