@@ -289,9 +289,12 @@ public:
     void parse(FILE *in = stdin);
 
     void print(FILE *out);
+    void print(ostream &out);
     void print_constrained(FILE *out);
+    void print_constrained(ostream &out);
 
     void print_xml(FILE *out, bool constrained, const string &blob);
+    void print_xml(ostream &out, bool constrained, const string &blob);
 
     void mark_all(bool state);
     bool mark(const string &name, bool state);
