@@ -35,7 +35,7 @@
 #include "config.h"
 
 static char id[] not_used =
-    {"$Id: GSEClause.cc 16088 2007-03-28 21:42:19Z jimg $"
+    {"$Id: GSEClause.cc 16967 2007-08-20 23:39:01Z jimg $"
     };
 
 #include <iostream>
@@ -124,8 +124,12 @@ void
 GSEClause::set_start_stop()
 {
     // Read the map array, scan, set start and stop.
+#if 0
     T *vals = 0;
     d_map->buf2val((void **)&vals);
+#endif
+    T *vals = new T[d_map->length()];
+    d_map->value(vals);
 
     // Set the map's max and min values for use in error messages (it's a lot
     // easier to do here, now, than later... 9/20/2001 jhrg)
