@@ -27,7 +27,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    { "$Id: HTTPConnect.cc 16759 2007-06-26 19:37:30Z jimg $"
+    { "$Id: HTTPConnect.cc 16943 2007-08-15 20:15:39Z jimg $"
     };
 
 #include <stdio.h>
@@ -129,6 +129,7 @@ class ParseHeader : public unary_function<const string &, void>
     ObjectType type;  // What type of object is in the stream?
     string server;  // Server's version string.
     string protocol;            // Server's protocol version.
+    string location;            // Url returned by server
 
 public:
     ParseHeader() : type(unknown_type), server("dods/0.0"), protocol("2.0")
@@ -179,6 +180,12 @@ public:
             DBG2(cout << name << ": " << value << endl);
             server = value;
         }
+       	else if (name == "location:") {
+       	    string value; 
+       	    line >> value;
+       	    DBG2(cout << name << ": " << value << endl);
+       	    location = value;
+        }
         else if (type == unknown_type && name == "content-type:"
                  && line.str().find("text/html") != string::npos) {
             DBG2(cout << name << ": text/html..." << endl);
@@ -200,6 +207,10 @@ public:
     {
         return protocol;
     }
+
+    string get_location() {
+	   return location;
+    }
 };
 
 /** A libcurl callback function used to read response headers. Read headers,
@@ -547,6 +558,10 @@ HTTPConnect::~HTTPConnect()
 HTTPResponse *
 HTTPConnect::fetch_url(const string &url)
 {
+#ifdef HTTP_TRACE
+    cout << "GET " << url << " HTTP/1.0" << endl;
+#endif
+
     HTTPResponse *stream;
 
     if (d_http_cache && d_http_cache->is_cache_enabled()) {
@@ -556,11 +571,30 @@ HTTPConnect::fetch_url(const string &url)
         stream = plain_fetch_url(url);
     }
 
+#ifdef HTTP_TRACE
+    stringstream ss;
+    ss << "HTTP/1.0 " << stream->get_status() << " -" << endl;
+    for (size_t i = 0; i < stream->get_headers()->size(); i++) {
+	ss << stream->get_headers()->at(i) << endl;
+    }
+    cout << ss.str();
+#endif
+
     ParseHeader parser;
 
     parser = for_each(stream->get_headers()->begin(),
                       stream->get_headers()->end(), ParseHeader());
 
+#ifdef HTTP_TRACE
+    cout << endl << endl;
+#endif
+
+    // handle redirection case (2007-04-27, gaffigan@sfos.uaf.edu)
+    if (parser.get_location() != "" &&
+	url.substr(0,url.find("?",0)).compare(parser.get_location().substr(0,url.find("?",0))) != 0) {
+           return fetch_url(parser.get_location());
+    }
+
     stream->set_type(parser.get_object_type());
     stream->set_version(parser.get_server());
     stream->set_protocol(parser.get_protocol());
@@ -653,7 +687,7 @@ HTTPConnect::caching_fetch_url(const string &url)
 
             vector<string> *headers = new vector<string>;;
             FILE *s = d_http_cache->get_cached_response(url, *headers);
-            HTTPCacheResponse *crs = new HTTPCacheResponse(s, headers, d_http_cache);
+            HTTPCacheResponse *crs = new HTTPCacheResponse(s, 200, headers, d_http_cache);
 
             return crs;
         }
@@ -683,7 +717,7 @@ HTTPConnect::caching_fetch_url(const string &url)
                     DBGN(cerr << "read a new response; caching." << endl);
 
                     d_http_cache->cache_response(url, now, *resp_hdrs, body);
-                    HTTPResponse *rs = new HTTPResponse(body, resp_hdrs,
+                    HTTPResponse *rs = new HTTPResponse(body, http_status, resp_hdrs,
                                                         dods_temp);
 
                     return rs;
@@ -699,7 +733,7 @@ HTTPConnect::caching_fetch_url(const string &url)
                     vector<string> *headers = new vector<string>;;
                     FILE *s = d_http_cache->get_cached_response(url, *headers);
                     HTTPCacheResponse *crs
-                    = new HTTPCacheResponse(s, headers, d_http_cache);
+                    = new HTTPCacheResponse(s, 304, headers, d_http_cache);
                     return crs;
                 }
                 break;
@@ -754,8 +788,9 @@ HTTPConnect::plain_fetch_url(const string &url)
     string dods_temp = get_temp_file(stream);
     vector<string> *resp_hdrs = new vector<string>;
 
+    int status = -1;
     try {
-        int status = read_url(url, stream, resp_hdrs); // Throws Error.
+        status = read_url(url, stream, resp_hdrs); // Throws Error.
         if (status >= 400) {
             string msg = "Error while reading the URL: ";
             msg += url;
@@ -772,7 +807,7 @@ HTTPConnect::plain_fetch_url(const string &url)
 
     rewind(stream);
 
-    return new HTTPResponse(stream, resp_hdrs, dods_temp);
+    return new HTTPResponse(stream, status, resp_hdrs, dods_temp);
 }
 
 /** Set the <em>accept deflate</em> property. If true, the DAP client
