@@ -80,13 +80,14 @@ public:
     memory), the temp file should be set to "".
 
     @param s FILE * to the response. Read the response body from this
-    stream. 
+    stream.
+    @param status The HTTP response status code.
     @param h Response headers. This class will delete the pointer when
     the instance that contains it is destroyed.
     @param temp_file Name a the temporary file that holds the response
     body; this file is deleted when this instance is deleted. */
-    HTTPResponse(FILE *s, vector<string> *h, const string &temp_file)
-            : Response(s), d_headers(h), d_file(temp_file)
+    HTTPResponse(FILE *s, int status, vector<string> *h, const string &temp_file)
+            : Response(s, status), d_headers(h), d_file(temp_file)
     {
         DBG(cerr << "Headers: " << endl);
         DBGN(copy(d_headers->begin(), d_headers->end(),
