@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 #ifndef _object_type_h
 #define _object_type_h
 
@@ -39,7 +39,8 @@
        dods_dds,
        dods_data,
        dods_error,
-       web_error
+       web_error,
+       dap4_ddx
      };
      </pre>
 
@@ -52,7 +53,8 @@ enum ObjectType {
     dods_dds,
     dods_data,
     dods_error,
-    web_error
+    web_error,
+    dap4_ddx
 };
 
 #endif
