@@ -37,7 +37,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: RValue.cc 16088 2007-03-28 21:42:19Z jimg $"
+    {"$Id: RValue.cc 17156 2007-09-21 17:44:27Z jimg $"
     };
 
 #include <assert.h>
@@ -48,6 +48,8 @@ static char rcsid[] not_used =
 #include "expr.h"
 #include "RValue.h"
 #include "DDS.h"
+#include "dods-limits.h"
+#include "util.h"
 
 using namespace std;
 
@@ -96,17 +98,27 @@ build_btp_args(rvalue_list *args, DDS &dds, const string &dataset)
 
     if (args)
         argc = args->size();
+        
+    // Sanitize allocation size
+    if (!size_ok(sizeof(BaseType*), argc+1))
+    	throw Error(malformed_expr, 
+                string("Malformed argument list (") 
+                + long_to_string(argc)  
+                + string(")."));
 
     // Add space for a null terminator
-    BaseType **argv = new BaseType *[argc + 1];
+    BaseType **argv = new BaseType*[argc + 1];
 
     int index = 0;
-    if (argc) {
-        for (rvalue::Args_iter i = args->begin(); i != args->end(); i++) {
+    if (argv && argc) {
+        for (rvalue::Args_iter i = args->begin(); i != args->end() && index < argc+1; ++i) {
             argv[index++] = (*i)->bvalue(dataset, dds);
         }
     }
 
+	if (index > argc)
+		throw InternalErr(__FILE__, __LINE__, "index out of range.");
+		
     argv[index] = 0;            // Add the null terminator.
 
     return argv;
