@@ -40,6 +40,7 @@
 #include <sstream>
 
 //#define DODS_DEBUG
+//#define DODS_DEBUG2
 
 #include "debug.h"
 #include "Error.h"
@@ -100,22 +101,22 @@ Sequence::_duplicate(const Sequence &s)
 }
 
 static void
-write_end_of_sequence(XDR *sink)
+write_end_of_sequence(Marshaller &m)
 {
-    xdr_opaque(sink, (char *)&end_of_sequence, 1);
+    m.put_opaque( (char *)&end_of_sequence, 1 ) ;
 }
 
 static void
-write_start_of_instance(XDR *sink)
+write_start_of_instance(Marshaller &m)
 {
-    xdr_opaque(sink, (char *)&start_of_instance, 1);
+    m.put_opaque( (char *)&start_of_instance, 1 ) ;
 }
 
 static unsigned char
-read_marker(XDR *source)
+read_marker(UnMarshaller &um)
 {
     unsigned char marker;
-    xdr_opaque(source, (char *)&marker, 1);
+    um.get_opaque( (char *)&marker, 1 ) ;
 
     return marker;
 }
@@ -308,17 +309,11 @@ Sequence::add_var(BaseType *bt, Part)
         throw InternalErr(__FILE__, __LINE__,
                           "Cannot add variable: NULL pointer");
     // Jose Garcia
-    // We append a copy of bt so the owner
-    // of bt is free to deallocate as he wishes.
-    DBG2(cerr << "In Sequence::add_var(), bt: " << bt << endl);
-    DBG2(cerr << bt->toString() << endl);
+    // We append a copy of bt so the owner of bt is free to deallocate
 
     BaseType *bt_copy = bt->ptr_duplicate();
     bt_copy->set_parent(this);
     _vars.push_back(bt_copy);
-
-    DBG2(cerr << "In Sequence::add_var(), bt_copy: " << bt_copy << endl);
-    DBG2(cerr << bt_copy->toString() << endl);
 }
 
 // Deprecated
@@ -555,6 +550,15 @@ Sequence::reset_row_number()
 
     @note The first row is row number zero. A Sequence with 100 rows will
     have row numbers 0 to 99.
+    
+    @todo This code ignores the main reason for nesting the sequences, that
+    if the outer Sequence's current instance fails the CE, there's no need to
+    look at the values of the inner Sequence. But in the code that calls this
+    method (serialize() and intern_data()) the CE is not evaluated until the
+    inner-most Sequence (i.e., the leaf Sequence) is read. That means that
+    each instance of the inner Sequence is read and the CE evaluated for each
+    of those reads. To fix this, and the overall problem of complexity here,
+    we need to re-think Sequences and how they behave. 11/13/2007 jhrg
 
     @return A boolean value, with TRUE indicating that read_row
     should be called again because there's more data to be read.
@@ -633,9 +637,9 @@ Sequence::is_end_of_rows(int i)
     <pre>
     Dataset {
         Sequence {
-     Int x;
-     Int y;
- } flat;
+            Int x;
+            Int y;
+        } flat;
     } case_1;
     </pre>
 
@@ -648,11 +652,11 @@ Sequence::is_end_of_rows(int i)
     <pre>
     Dataset {
         Sequence {
-     Int t;
-     Sequence {
-         Int z;
-     } inner;
- } outer;
+            Int t;
+            Sequence {
+                Int z;
+            } inner;
+        } outer;
     } case_2;
     </pre>
 
@@ -663,7 +667,7 @@ Sequence::is_end_of_rows(int i)
     SOI marker and values for outer [serialize_parent_part_two], then write
     the SOI and values for the first row of inner. Continue to read and send
     rows of inner until the last row has been read. Send EOS for inner
-    [serialize_leaf]. Now read the next row of outer and repeat. Once outer\
+    [serialize_leaf]. Now read the next row of outer and repeat. Once outer
     is completely read, send its EOS marker.</li>
     </ol>
 
@@ -683,15 +687,15 @@ Sequence::is_end_of_rows(int i)
 */
 bool
 Sequence::serialize(const string &dataset, ConstraintEvaluator &eval, DDS &dds,
-                    XDR *sink, bool ce_eval)
+                    Marshaller &m, bool ce_eval)
 {
     DBG2(cerr << "Entering Sequence::serialize for " << name() << endl);
 
     // Special case leaf sequences!
     if (is_leaf_sequence())
-        return serialize_leaf(dataset, dds, eval, sink, ce_eval);
+        return serialize_leaf(dataset, dds, eval, m, ce_eval);
     else
-        return serialize_parent_part_one(dataset, dds, eval, sink);
+        return serialize_parent_part_one(dataset, dds, eval, m);
 }
 
 // We know this is not a leaf Sequence. That means that this Sequence holds
@@ -700,7 +704,7 @@ Sequence::serialize(const string &dataset, ConstraintEvaluator &eval, DDS &dds,
 
 bool
 Sequence::serialize_parent_part_one(const string &dataset, DDS &dds,
-                                    ConstraintEvaluator &eval, XDR *sink)
+                                    ConstraintEvaluator &eval, Marshaller &m)
 {
     DBG2(cerr << "Entering serialize_parent_part_one for " << name() << endl);
 
@@ -731,7 +735,7 @@ Sequence::serialize_parent_part_one(const string &dataset, DDS &dds,
             // sequence must be the lowest level sequence with values whose send_p
             // property is true.
             if ((*iter)->send_p() && (*iter)->type() == dods_sequence_c)
-                (*iter)->serialize(dataset, eval, dds, sink);
+                (*iter)->serialize(dataset, eval, dds, m);
         }
 
         set_read_p(false); // ...so this will read the next instance
@@ -747,7 +751,7 @@ Sequence::serialize_parent_part_one(const string &dataset, DDS &dds,
     // a return value of only the EOS marker for the outermost sequence.
     if (d_top_most || d_wrote_soi) {
         DBG(cerr << "Writing End of Sequence marker" << endl);
-        write_end_of_sequence(sink);
+        write_end_of_sequence(m);
         d_wrote_soi = false;
     }
 
@@ -766,19 +770,19 @@ Sequence::serialize_parent_part_one(const string &dataset, DDS &dds,
 // variables.
 void
 Sequence::serialize_parent_part_two(const string &dataset, DDS &dds,
-                                    ConstraintEvaluator &eval, XDR *sink)
+                                    ConstraintEvaluator &eval, Marshaller &m)
 {
     DBG(cerr << "Entering serialize_parent_part_two for " << name() << endl);
 
     BaseType *btp = get_parent();
     if (btp && btp->type() == dods_sequence_c)
         dynamic_cast<Sequence&>(*btp).serialize_parent_part_two(dataset, dds,
-                eval, sink);
+                eval, m);
 
     if (d_unsent_data) {
         DBG(cerr << "Writing Start of Instance marker" << endl);
         d_wrote_soi = true;
-        write_start_of_instance(sink);
+        write_start_of_instance(m);
 
         // In this loop serialize will signal an error with an exception.
         for (Vars_iter iter = _vars.begin(); iter != _vars.end(); iter++) {
@@ -787,7 +791,7 @@ Sequence::serialize_parent_part_two(const string &dataset, DDS &dds,
                 << (*iter)->name() << endl);
             if ((*iter)->send_p() && (*iter)->type() != dods_sequence_c) {
                 DBG(cerr << "Send P is true, sending " << (*iter)->name() << endl);
-                (*iter)->serialize(dataset, eval, dds, sink, false);
+                (*iter)->serialize(dataset, eval, dds, m, false);
             }
         }
 
@@ -799,7 +803,7 @@ Sequence::serialize_parent_part_two(const string &dataset, DDS &dds,
 // is also a leaf sequence.
 bool
 Sequence::serialize_leaf(const string &dataset, DDS &dds,
-                         ConstraintEvaluator &eval, XDR *sink, bool ce_eval)
+                         ConstraintEvaluator &eval, Marshaller &m, bool ce_eval)
 {
     DBG(cerr << "Entering Sequence::serialize_leaf for " << name() << endl);
     int i = (d_starting_row_number != -1) ? d_starting_row_number : 0;
@@ -825,8 +829,7 @@ Sequence::serialize_leaf(const string &dataset, DDS &dds,
         BaseType *btp = get_parent();
         if (btp && btp->type() == dods_sequence_c)
             dynamic_cast<Sequence&>(*btp).serialize_parent_part_two(dataset,
-                    dds, eval,
-                    sink);
+                    dds, eval, m);
     }
 
     d_wrote_soi = false;
@@ -835,7 +838,7 @@ Sequence::serialize_leaf(const string &dataset, DDS &dds,
 
         DBG(cerr << "Writing Start of Instance marker" << endl);
         d_wrote_soi = true;
-        write_start_of_instance(sink);
+        write_start_of_instance(m);
 
         // In this loop serialize will signal an error with an exception.
         for (Vars_iter iter = _vars.begin(); iter != _vars.end(); iter++) {
@@ -843,7 +846,7 @@ Sequence::serialize_leaf(const string &dataset, DDS &dds,
                 << (*iter)->name() << endl);
             if ((*iter)->send_p()) {
                 DBG(cerr << "Send P is true, sending " << (*iter)->name() << endl);
-                (*iter)->serialize(dataset, eval, dds, sink, false);
+                (*iter)->serialize(dataset, eval, dds, m, false);
             }
         }
 
@@ -857,7 +860,7 @@ Sequence::serialize_leaf(const string &dataset, DDS &dds,
     // Marker in the stream.
     if (d_wrote_soi || d_top_most) {
         DBG(cerr << "Writing End of Sequence marker" << endl);
-        write_end_of_sequence(sink);
+        write_end_of_sequence(m);
     }
 
     return true;  // Signal errors with exceptions.
@@ -887,90 +890,47 @@ Sequence::serialize_leaf(const string &dataset, DDS &dds,
     @param eval Use this contraint evaluator
     @param dds This DDS holds the variables for the data source */
 void
-Sequence::transfer_data(const string &dataset,
-                        ConstraintEvaluator &eval,
-                        DDS &dds)
+Sequence::intern_data(const string &dataset, ConstraintEvaluator &eval, DDS &dds)
 {
-    DBG(cerr << "Entering Sequence::transfer_data for " << name() << endl);
-
-    DBG(cerr << "Sequence::transfer_data, values: " << &d_values << endl);
+    DBG(cerr << "Entering intern_data for " << name() << endl);
+    DBG2(cerr << "    intern_data, values: " << &d_values << endl);
 
+    // Why use a stack instead of return values? We need the stack because
+    // Sequences neted three of more levels deep will loose the middle
+    // instances when the intern_data_parent_part_two() code is run.
     sequence_values_stack_t sequence_values_stack;
-    sequence_values_stack.push_back(&d_values);
+    
+    DBG2(cerr << "    pushing d_values (" << &d_values
+         << ") on stack; size: " << sequence_values_stack.size() << endl);
+    sequence_values_stack.push(&d_values);
 
-    transfer_data_private(dataset, eval, dds, sequence_values_stack);
+    intern_data_private(dataset, eval, dds, sequence_values_stack);
 }
 
 void
-Sequence::transfer_data_private(const string &dataset,
+Sequence::intern_data_private(const string &dataset,
                                 ConstraintEvaluator &eval,
                                 DDS &dds,
                                 sequence_values_stack_t &sequence_values_stack)
 {
-    DBG(cerr << "Entering Sequence::transfer_data_private for " << name() << endl);
+    DBG(cerr << "Entering intern_data_private for " << name() << endl);
 
-    if (is_leaf_sequence()) {
-        transfer_data_for_leaf(dataset, dds, eval, sequence_values_stack);
-    }
-    else {
-        transfer_data_parent_part_one(dataset, dds, eval, sequence_values_stack);
-    }
+    if (is_leaf_sequence())
+        intern_data_for_leaf(dataset, dds, eval, sequence_values_stack);
+    else
+        intern_data_parent_part_one(dataset, dds, eval, sequence_values_stack);
 }
 
 void
-Sequence::transfer_data_for_leaf(const string &dataset, DDS &dds,
-                                 ConstraintEvaluator &eval,
-                                 sequence_values_stack_t &sequence_values_stack)
-{
-    int i = (get_starting_row_number() != -1) ? get_starting_row_number() : 0;
-
-    bool status = read_row(i, dataset, dds, eval, true);
-
-    if (status && (get_ending_row_number() == -1 || i <= get_ending_row_number())) {
-        BaseType *btp = get_parent();
-        if (btp && btp->type() == dods_sequence_c) {
-            dynamic_cast<Sequence&>(*btp).transfer_data_parent_part_two(dataset, dds, eval, sequence_values_stack);
-        }
-    }
-
-    SequenceValues *values = sequence_values_stack.back();
-    DBG(cerr << "In td_for_leaf, using values = " << values << endl);
-
-    while (status && (get_ending_row_number() == -1 || i <= get_ending_row_number())) {
-        i += get_row_stride();
-
-        BaseTypeRow *row_data = new BaseTypeRow;
-
-        // In this loop serialize will signal an error with an exception.
-        for (Vars_iter iter = var_begin(); iter != var_end(); iter++) {
-            if ((*iter)->send_p()) {
-                row_data->push_back((*iter)->ptr_duplicate());
-            }
-        }
-
-        DBG(cerr << "Pushing values for " << name() << " to " << values << endl);
-        // Save the row_data to the values().
-        values->push_back(row_data);
-        set_read_p(false);      // ...so this will read the next instance
-        status = read_row(i, dataset, dds, eval, true);
-    }
-
-    sequence_values_stack.pop_back();
-    DBG(cerr << "In td_for_leaf, Poping the top of the stack" << endl);;
-}
-
-
-void
-Sequence::transfer_data_parent_part_one(const string & dataset, DDS & dds,
+Sequence::intern_data_parent_part_one(const string & dataset, DDS & dds,
                                         ConstraintEvaluator & eval,
                                         sequence_values_stack_t &
                                         sequence_values_stack)
 {
-    DBG(cerr << "Entering transfer_data_parent_part_one for " << name() <<
+    DBG(cerr << "Entering intern_data_parent_part_one for " << name() <<
         endl);
 
-    int i =
-        (get_starting_row_number() != -1) ? get_starting_row_number() : 0;
+    int i = (get_starting_row_number() != -1) ? get_starting_row_number() : 0;
 
     // read_row returns true if valid data was read, false if the EOF was
     // found. 6/1/2001 jhrg
@@ -986,20 +946,12 @@ Sequence::transfer_data_parent_part_one(const string & dataset, DDS & dds,
             if ((*iter)->send_p()) {
                 switch ((*iter)->type()) {
                 case dods_sequence_c:
-                    dynamic_cast <
-                    Sequence & >(**iter).transfer_data_private(dataset,
-                            eval,
-                            dds,
-                            sequence_values_stack);
-                    break;
-
-                case dods_structure_c:
-                    dynamic_cast <
-                    Structure & >(**iter).transfer_data(dataset, eval, dds);
+                    dynamic_cast<Sequence&>(**iter).intern_data_private(dataset,
+                            eval, dds, sequence_values_stack);
                     break;
 
                 default:
-                    (*iter)->read(dataset);
+                    (*iter)->intern_data(dataset, eval, dds);
                     break;
                 }
             }
@@ -1012,25 +964,27 @@ Sequence::transfer_data_parent_part_one(const string & dataset, DDS & dds,
 
     // Reset current row number for next nested sequence element.
     reset_row_number();
-    sequence_values_stack.pop_back();
-    DBG(cerr << "In td_fparent_part_one, Poping the top of the stack" <<
-        endl);;
+    DBG2(cerr << "    popping d_values (" << sequence_values_stack.top()
+         << ") off stack; size: " << sequence_values_stack.size() << endl);
+    sequence_values_stack.pop();
 }
 
 void
-Sequence::transfer_data_parent_part_two(const string &dataset, DDS &dds,
-                                        ConstraintEvaluator &eval,
-                                        sequence_values_stack_t &sequence_values_stack)
+Sequence::intern_data_parent_part_two(const string &dataset, DDS &dds,
+                                      ConstraintEvaluator &eval,
+                                      sequence_values_stack_t &sequence_values_stack)
 {
-    DBG(cerr << "Entering transfer_data_parent_part_two for " << name() << endl);
+    DBG(cerr << "Entering intern_data_parent_part_two for " << name() << endl);
 
     BaseType *btp = get_parent();
     if (btp && btp->type() == dods_sequence_c) {
-        dynamic_cast<Sequence&>(*btp).transfer_data_parent_part_two(dataset, dds, eval, sequence_values_stack);
+        dynamic_cast<Sequence&>(*btp).intern_data_parent_part_two(dataset, 
+                                      dds, eval, sequence_values_stack);
     }
 
-    SequenceValues *values = sequence_values_stack.back();
-    DBG(cerr << "In td_parent_part_two, using values = " << values << endl);
+    DBG2(cerr << "    stack size: " << sequence_values_stack.size() << endl);
+    SequenceValues *values = sequence_values_stack.top();
+    DBG2(cerr << "    using values = " << values << endl);
 
     if (get_unsent_data()) {
         BaseTypeRow *row_data = new BaseTypeRow;
@@ -1041,21 +995,79 @@ Sequence::transfer_data_parent_part_two(const string &dataset, DDS &dds,
             if ((*iter)->send_p() && (*iter)->type() != dods_sequence_c) {
                 row_data->push_back((*iter)->ptr_duplicate());
             }
-            else if ((*iter)->send_p()) { //Sequence, and it must be last the variable
+            else if ((*iter)->send_p()) { //Sequence; must be the last variable
                 Sequence *tmp = dynamic_cast<Sequence*>((*iter)->ptr_duplicate());
                 row_data->push_back(tmp);
-                DBG(cerr << "In td_parent_part_two, pushing values = "
-                    << &(dynamic_cast<Sequence&>(*tmp).d_values) << endl);
-                sequence_values_stack.push_back(&(dynamic_cast<Sequence&>(*tmp).d_values));
+                DBG2(cerr << "    pushing d_values (" << &(dynamic_cast<Sequence&>(*tmp).d_values)
+                     << ") on stack; size: " << sequence_values_stack.size() << endl);
+                // This pushes the d_values field of the newly created leaf 
+                // Sequence onto the stack. The code then returns to intern
+                // _data_for_leaf() where this value will be used.
+                sequence_values_stack.push(&(dynamic_cast<Sequence&>(*tmp).d_values));
             }
         }
 
-        DBG(cerr << "Pushing values for " << name() << " to " << values << endl);
+        DBG2(cerr << "    pushing values for " << name() << " to " << values << endl);
         values->push_back(row_data);
         set_unsent_data(false);
     }
 }
 
+void
+Sequence::intern_data_for_leaf(const string &dataset, DDS &dds,
+                               ConstraintEvaluator &eval,
+                               sequence_values_stack_t &sequence_values_stack)
+{
+    DBG(cerr << "Entering intern_data_for_leaf for " << name() << endl);
+    
+    int i = (get_starting_row_number() != -1) ? get_starting_row_number() : 0;
+
+    DBG2(cerr << "    reading row " << i << endl);
+    bool status = read_row(i, dataset, dds, eval, true);
+    DBG2(cerr << "    status: " << status << endl);
+    
+    if (status && (get_ending_row_number() == -1 || i <= get_ending_row_number())) {
+        BaseType *btp = get_parent();
+        if (btp && btp->type() == dods_sequence_c) {
+            // This call will read the values for the parent sequences and 
+            // then allocate a new instance for the leaf and push that onto
+            // the stack.
+            dynamic_cast<Sequence&>(*btp).intern_data_parent_part_two(dataset,
+                dds, eval, sequence_values_stack);
+        }
+
+        // intern_data_parent_part_two pushes the d_values field of the leaf
+        // onto the stack, so this operation grabs that value and then loads
+        // data into it.
+        SequenceValues *values = sequence_values_stack.top();
+        DBG2(cerr << "    using values = " << values << endl);
+
+        while (status && (get_ending_row_number() == -1 
+                          || i <= get_ending_row_number())) {
+            i += get_row_stride();
+
+            // Copy data from the object's fields to this new BaeTypeRow instance
+            BaseTypeRow *row_data = new BaseTypeRow;
+            for (Vars_iter iter = var_begin(); iter != var_end(); iter++) {
+                if ((*iter)->send_p()) {
+                    row_data->push_back((*iter)->ptr_duplicate());
+                }
+            }
+
+            DBG2(cerr << "    pushing values for " << name() << " to " << values << endl);
+            // Save the row_data to values().
+            values->push_back(row_data);
+            
+            set_read_p(false);      // ...so this will read the next instance
+            // Read the ith row into this object's fields
+            status = read_row(i, dataset, dds, eval, true);
+        }
+
+        DBG2(cerr << "    popping d_values (" << sequence_values_stack.top()
+             << ") off stack; size: " << sequence_values_stack.size() << endl);
+        sequence_values_stack.pop();
+    }
+}
 
 /** @brief Deserialize (read from the network) the entire Sequence.
 
@@ -1063,19 +1075,22 @@ Sequence::transfer_data_parent_part_two(const string &dataset, DDS &dds,
     sequence is read at once. The method used to return True to indicate
     that more data needed to be deserialized and False when the sequence
     was completely read. Now it simply returns false. This might seem odd,
-    but making this method return true breaks existing software the least.
+    but making this method return false breaks existing software the least.
 
-    @param source An XDR stream.
+    @param um An UnMarshaller that knows how to deserialize data
     @param dds A DataDDS from which to read.
-    @param reuse Passed to child objects when they are deserialized.
+    @param reuse Passed to child objects when they are deserialized. Some 
+    implementations of derialize() use this to determine if new storage should
+    be allocated or existing storage reused.
     @exception Error if a sequence stream marker cannot be read.
     @exception InternalErr if the <tt>dds</tt> param is not a DataDDS.
     @return A return value of false indicates that an EOS ("end of
     Sequence") marker was found, while a value of true indicates
-    that there are more rows to be read.
+    that there are more rows to be read. This version always reads the 
+    entire sequence, so it always returns false.
 */
 bool
-Sequence::deserialize(XDR *source, DDS *dds, bool reuse)
+Sequence::deserialize(UnMarshaller &um, DDS *dds, bool reuse)
 {
     DataDDS *dd = dynamic_cast<DataDDS *>(dds);
     if (!dd)
@@ -1093,7 +1108,7 @@ Sequence::deserialize(XDR *source, DDS *dds, bool reuse)
 
     while (true) {
         // Grab the sequence stream's marker.
-        unsigned char marker = read_marker(source);
+        unsigned char marker = read_marker(um);
         if (is_end_of_sequence(marker))
             break;  // EXIT the while loop here!!!
         else if (is_start_of_instance(marker)) {
@@ -1104,7 +1119,7 @@ Sequence::deserialize(XDR *source, DDS *dds, bool reuse)
             // Read the instance's values, building up the row
             for (Vars_iter iter = _vars.begin(); iter != _vars.end(); iter++) {
                 BaseType *bt_ptr = (*iter)->ptr_duplicate();
-                bt_ptr->deserialize(source, dds, reuse);
+                bt_ptr->deserialize(um, dds, reuse);
                 DBG2(cerr << "Deserialized " << bt_ptr->name() << " ("
                      << bt_ptr << ") = ");
                 DBG2(bt_ptr->print_val(stderr, ""));
@@ -1249,6 +1264,81 @@ Sequence::print_one_row(FILE *out, int row, string space,
     fprintf(out, " }") ;
 }
 
+void
+Sequence::print_one_row(ostream &out, int row, string space,
+                        bool print_row_num)
+{
+    if (print_row_num)
+	out << "\n" << space << row << ": " ;
+
+    out << "{ " ;
+
+    int elements = element_count();
+    int j = 0;
+    BaseType *bt_ptr = 0;
+    
+    // This version of print_one_row() works for both data read with 
+    // deserialize(), where each variable is assumed to have valid data, and
+    // intern_data(), where some/many variables do not. Because of that, it's
+    // not correct to assume that all of the elements will be printed, which 
+    // is what the old code did.
+    // Print the first value
+    while (j < elements && !bt_ptr) {
+        bt_ptr = var_value(row, j++);
+        if (bt_ptr) {  // data
+            if (bt_ptr->type() == dods_sequence_c)
+                dynamic_cast<Sequence*>(bt_ptr)->print_val_by_rows
+                     (out, space + "    ", false, print_row_num);
+            else
+                bt_ptr->print_val(out, space, false);
+        }
+    }
+
+    // Print the remaining values
+    while (j < elements) {
+        bt_ptr = var_value(row, j++);
+        if (bt_ptr) {  // data
+            out << ", ";
+            if (bt_ptr->type() == dods_sequence_c)
+                dynamic_cast<Sequence*>(bt_ptr)->print_val_by_rows
+                        (out, space + "    ", false, print_row_num);
+            else
+                bt_ptr->print_val(out, space, false);
+        }
+    }
+    
+#if 0
+    // old version
+    int elements = element_count() - 1;
+    int j;
+    BaseType *bt_ptr;
+    // Print first N-1 elements of the row.
+    for (j = 0; j < elements; ++j) {
+        bt_ptr = var_value(row, j);
+        if (bt_ptr) {  // data
+            if (bt_ptr->type() == dods_sequence_c)
+                dynamic_cast<Sequence*>(bt_ptr)->print_val_by_rows
+                     (out, space + "    ", false, print_row_num);
+            else
+                bt_ptr->print_val(out, space, false);
+	    out << ", " ;
+        }
+    }
+
+    // Print Nth element; end with a `}.'
+    bt_ptr = var_value(row, j);
+    if (bt_ptr) {  // data
+        if (bt_ptr->type() == dods_sequence_c)
+            dynamic_cast<Sequence*>(bt_ptr)->print_val_by_rows
+            (out, space + "    ", false, print_row_num);
+        else
+            bt_ptr->print_val(out, space, false);
+    }
+#endif
+
+    out << " }" ;
+}
+
 void
 Sequence::print_val_by_rows(FILE *out, string space, bool print_decl_p,
                             bool print_row_numbers)
@@ -1274,26 +1364,44 @@ Sequence::print_val_by_rows(FILE *out, string space, bool print_decl_p,
         fprintf(out, ";\n") ;
 }
 
+void
+Sequence::print_val_by_rows(ostream &out, string space, bool print_decl_p,
+                            bool print_row_numbers)
+{
+    if (print_decl_p) {
+        print_decl(out, space, false);
+	out << " = " ;
+    }
+
+    out << "{ " ;
+
+    int rows = number_of_rows() - 1;
+    int i;
+    for (i = 0; i < rows; ++i) {
+        print_one_row(out, i, space, print_row_numbers);
+	out << ", " ;
+    }
+    print_one_row(out, i, space, print_row_numbers);
+
+    out << " }" ;
+
+    if (print_decl_p)
+	out << ";\n" ;
+}
+
 void
 Sequence::print_val(FILE *out, string space, bool print_decl_p)
 {
     print_val_by_rows(out, space, print_decl_p, false);
 }
 
-
-// print_all_vals is from Todd Karakasian.
-// We need to integrate this into print_val somehow, maybe by adding an XDR *
-// to Sequence? This can wait since print_val is mostly used for debugging...
-//
-// Deprecated. No longer needed since print_vals does its job.
-
 void
-Sequence::print_all_vals(FILE *out, XDR *, DDS *, string space,
-                         bool print_decl_p)
+Sequence::print_val(ostream &out, string space, bool print_decl_p)
 {
-    print_val(out, space, print_decl_p);
+    print_val_by_rows(out, space, print_decl_p, false);
 }
 
+
 bool
 Sequence::check_semantics(string &msg, bool all)
 {
@@ -1400,22 +1508,22 @@ Sequence::dump(ostream &strm) const
     DapIndent::Indent() ;
     Constructor::dump(strm) ;
     strm << DapIndent::LMarg << "# rows deserialized: " << d_row_number
-    << endl ;
+         << endl ;
     strm << DapIndent::LMarg << "bracket notation information:" << endl ;
     DapIndent::Indent() ;
     strm << DapIndent::LMarg << "starting row #: " << d_starting_row_number
-    << endl ;
+         << endl ;
     strm << DapIndent::LMarg << "row stride: " << d_row_stride << endl ;
     strm << DapIndent::LMarg << "ending row #: " << d_ending_row_number
-    << endl ;
+         << endl ;
     DapIndent::UnIndent() ;
 
     strm << DapIndent::LMarg << "data been sent? " << d_unsent_data << endl ;
     strm << DapIndent::LMarg << "start of instance? " << d_wrote_soi << endl ;
     strm << DapIndent::LMarg << "is leaf sequence? " << d_leaf_sequence
-    << endl ;
+         << endl ;
     strm << DapIndent::LMarg << "top most in hierarchy? " << d_top_most
-    << endl ;
+         << endl ;
     DapIndent::UnIndent() ;
 }
 
