@@ -208,6 +208,29 @@ Structure::add_var(BaseType *bt, Part)
     _vars.push_back(btp);
 }
 
+/** @brief simple implementation of reat that iterates through vars
+ *  and calls read on them
+ *
+ * @param dataset A string naming the dataset from which the data is to
+ * be read. The meaning of this string will vary among different types of
+ * data sources. It \e may be the name of a data file or an identifier
+ * used to read data from a relational database.
+ * @return returns false to signify all has been read
+ */
+bool
+Structure::read(const string &dataset)
+{
+    if( !read_p() )
+    {
+	for (Vars_iter i = _vars.begin(); i != _vars.end(); i++) {
+	    (*i)->read(dataset) ;
+	}
+	set_read_p(true) ;
+    }
+
+    return false ;
+}
+
 unsigned int
 Structure::width()
 {
@@ -221,36 +244,21 @@ Structure::width()
 }
 
 void
-Structure::transfer_data(const string & dataset,
-                         ConstraintEvaluator & eval, DDS & dds)
+Structure::intern_data(const string & dataset, ConstraintEvaluator & eval, DDS & dds)
 {
     if (!read_p())
         read(dataset);          // read() throws Error and InternalErr
 
     for (Vars_iter i = _vars.begin(); i != _vars.end(); i++) {
         if ((*i)->send_p()) {
-            switch ((*i)->type()) {
-            case dods_sequence_c:
-                dynamic_cast <Sequence & >(**i).transfer_data(dataset,
-                        eval, dds);
-                break;
-
-            case dods_structure_c:
-                dynamic_cast <Structure & >(**i).transfer_data(dataset,
-                        eval, dds);
-                break;
-
-            default:
-                (*i)->read(dataset);
-                break;
-            }
+            (*i)->intern_data(dataset, eval, dds);
         }
     }
 }
 
 bool
 Structure::serialize(const string &dataset, ConstraintEvaluator &eval, DDS &dds,
-                     XDR *sink, bool ce_eval)
+                     Marshaller &m, bool ce_eval)
 {
     dds.timeout_on();
 
@@ -261,11 +269,12 @@ Structure::serialize(const string &dataset, ConstraintEvaluator &eval, DDS &dds,
     if (ce_eval && !eval.eval_selection(dds, dataset))
         return true;
 #endif
+
     dds.timeout_off();
 
     for (Vars_iter i = _vars.begin(); i != _vars.end(); i++) {
         if ((*i)->send_p()) {
-            (*i)->serialize(dataset, eval, dds, sink, false);
+            (*i)->serialize(dataset, eval, dds, m, false);
         }
     }
 
@@ -273,10 +282,10 @@ Structure::serialize(const string &dataset, ConstraintEvaluator &eval, DDS &dds,
 }
 
 bool
-Structure::deserialize(XDR *source, DDS *dds, bool reuse)
+Structure::deserialize(UnMarshaller &um, DDS *dds, bool reuse)
 {
     for (Vars_iter i = _vars.begin(); i != _vars.end(); i++) {
-        (*i)->deserialize(source, dds, reuse);
+        (*i)->deserialize(um, dds, reuse);
     }
 
     return false;
@@ -418,9 +427,23 @@ Structure::print_val(FILE *out, string space, bool print_decl_p)
 }
 
 void
-Structure::print_all_vals(FILE *out, XDR *, DDS *, string space, bool print_decl_p)
+Structure::print_val(ostream &out, string space, bool print_decl_p)
 {
-    print_val(out, space, print_decl_p);
+    if (print_decl_p) {
+        print_decl(out, space, false);
+	out << " = " ;
+    }
+
+    out << "{ " ;
+    for (Vars_citer i = _vars.begin(); i != _vars.end();
+         i++, (void)(i != _vars.end() && out << ", ")) {
+        (*i)->print_val(out, "", false);
+    }
+
+    out << " }" ;
+
+    if (print_decl_p)
+	out << ";\n" ;
 }
 
 bool
