@@ -37,7 +37,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    { "$Id: Vector.cc 16466 2007-05-14 17:41:27Z jimg $"
+    { "$Id: Vector.cc 17489 2007-11-26 23:18:58Z pwest $"
     };
 
 //#define DODS_DEBUG
@@ -350,9 +350,9 @@ unsigned int Vector::width()
 // Returns: the number of elements in the vector.
 
 /** Returns the number of elements in the vector. Note that some
-child classes of Vector use the length of -1 as a flag value.
+    child classes of Vector use the length of -1 as a flag value.
 
-@see Vector::append_dim */
+    @see Vector::append_dim */
 int Vector::length() const
 {
     return _length;
@@ -382,12 +382,81 @@ void Vector::vec_resize(int l)
     _vec.resize((l > 0) ? l : 0, 0);    // Fill with NULLs
 }
 
+/** @brief read data into a variable for later use
+ 
+    Most uses of a variable are to either serialize its data to a stream of
+    some sort or to read values from some stream and intern those in the
+    variable for later use. These operations are perform by serialize()
+    and deserialize() which follow. This function performs essentially both
+    of these operations without actually using a stream device. The data are
+    read using the read() method(s) and loaded into the variables directly.
+    
+    This method is intended to be used by objects which transform DAP objects
+    like the DataDDS into an ASCII CSV representation.
+    
+    @param dataset A string passed to read() so data values can be read from
+    the data source.
+    @param eval A reference to a constraint evaluator
+    @param dds The complete DDS to which this variable belongs */
+void
+Vector::intern_data(const string &dataset, ConstraintEvaluator &eval, DDS &dds)
+{
+    if (!read_p())
+        read(dataset);          // read() throws Error and InternalErr
+
+    // length() is not capacity; it must be set explicitly in read().
+    int num = length();
+
+    switch (_var->type()) {
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:
+        // For these cases, read() puts the data into _buf, which is what we
+        // need to do 'stuff' with the data.
+        break;
+        
+    case dods_str_c:
+    case dods_url_c:
+        // For these cases, read() will put the data into d_str[], which is 
+        // what the transformation classes need.
+        break;
+
+    case dods_array_c:
+        // I think this is an error since there can never be an Array of 
+        // Array.
+        throw InternalErr(__FILE__, __LINE__, "Array of Array not supported.");
+        break;
+        
+    case dods_structure_c:
+    case dods_sequence_c:
+    case dods_grid_c:
+        // For these cases, we need to call read() for each of the 'num' 
+        // elements in the '_vec[]' array of BaseType object pointers.
+        if (_vec.capacity() == 0)
+            throw InternalErr(__FILE__, __LINE__,
+                              "The capacity of *this* vector is 0.");
+
+        for (int i = 0; i < num; ++i)
+            _vec[i]->intern_data(dataset, eval, dds);
+
+        break;
+
+    default:
+        throw InternalErr(__FILE__, __LINE__, "Unknown datatype.");
+        break;
+    }
+}
+
 /** @brief Serialize a Vector.
 
-    This uses the BaseType member xdr_coder to encode each element of
-    a cardinal array. See Sun's XDR manual. For Arrays of Str and Url
-    types, send the element count over as a prefix to the data so that
-    deserialize will know how many elements to read.
+    This uses the Marshaller class to encode each element of a cardinal
+    array. For Arrays of Str and Url types, send the element count over
+    as a prefix to the data so that deserialize will know how many elements
+    to read.
 
     NB: Arrays of cardinal types must already be in BUF (in the local machine's
     representation) <i>before</i> this call is made.
@@ -395,7 +464,7 @@ void Vector::vec_resize(int l)
 
 
 bool Vector::serialize(const string & dataset, ConstraintEvaluator & eval,
-                       DDS & dds, XDR * sink, bool ce_eval)
+                       DDS & dds, Marshaller &m, bool ce_eval)
 {
     int i = 0;
 
@@ -416,37 +485,15 @@ bool Vector::serialize(const string & dataset, ConstraintEvaluator & eval,
 
     switch (_var->type()) {
     case dods_byte_c:
+	m.put_vector( _buf, num, *this ) ;
+	break ;
     case dods_int16_c:
     case dods_uint16_c:
     case dods_int32_c:
     case dods_uint32_c:
     case dods_float32_c:
     case dods_float64_c:
-        // Jose Garcia
-        // If we are trying to serialize data and the internal buffer is
-        // unset, then the read method failed to do its job.
-        if (!_buf)
-            throw InternalErr(__FILE__, __LINE__,
-                              "Buffer pointer is not set.");
-
-        if ((0 == xdr_int(sink,  &num))) // send vector length
-            throw Error("Network I/O Error(1). This may be due to a bug in libdap or a\nproblem with the network connection.");
-
-        // Note that xdr_bytes and xdr_array both send the length themselves,
-        // so the length is actually sent twice. 12/31/99 jhrg
-        bool status;
-        if (_var->type() == dods_byte_c)
-            status = (0 != xdr_bytes(sink, (char **) & _buf,
-                                     (unsigned int *) & num,
-                                     DODS_MAX_ARRAY));
-        else
-            status = (0 != xdr_array(sink, (char **) & _buf,
-                                     (unsigned int *) & num,
-                                     DODS_MAX_ARRAY, _var->width(),
-                                     (xdrproc_t)(_var->xdr_coder())));
-        if (!status)
-            throw Error("Network I/O Error(2). This may be due to a bug in libdap or a\nproblem with the network connection.");
-
+	m.put_vector( _buf, num, _var->width(), *this ) ;
         break;
 
     case dods_str_c:
@@ -455,12 +502,10 @@ bool Vector::serialize(const string & dataset, ConstraintEvaluator & eval,
             throw InternalErr(__FILE__, __LINE__,
                               "The capacity of the string vector is 0");
 
-        if ((0 == xdr_int(sink, (int *) &num)))
-            throw Error("Network I/O Error(3). This may be due to a bug in libdap or a\nproblem with the network connection.");
+	m.put_int( num ) ;
 
         for (i = 0; i < num; ++i)
-            if (!xdr_str(sink, d_str[i]))
-                throw Error("Network I/O Error(4). Could not send string data.\nThis may be due to a bug in libdap, on the server or a\nproblem with the network connection.");
+	    m.put_str( d_str[i] ) ;
 
         break;
 
@@ -474,11 +519,10 @@ bool Vector::serialize(const string & dataset, ConstraintEvaluator & eval,
             throw InternalErr(__FILE__, __LINE__,
                               "The capacity of *this* vector is 0.");
 
-        if ((0 == xdr_int(sink, &num)))
-            throw Error("Network I/O Error. This may be due to a bug in libdap or a\nproblem with the network connection.");
+	m.put_int( num ) ;
 
         for (i = 0; i < num; ++i)
-            _vec[i]->serialize(dataset, eval, dds, sink, false);
+            _vec[i]->serialize(dataset, eval, dds, m, false);
 
         break;
 
@@ -507,9 +551,12 @@ bool Vector::serialize(const string & dataset, ConstraintEvaluator & eval,
 //
 // Returns: True is successful, false otherwise.
 
-bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
+bool Vector::deserialize(UnMarshaller &um, DDS * dds, bool reuse)
 {
+#if 0
+    // status no longer used. jhrg 8/28/07
     bool status;
+#endif
     unsigned int num;
     unsigned i = 0;
 
@@ -525,8 +572,7 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
             delete[]_buf;
         _buf = 0;
 
-        if ((0 == xdr_int(source, (int *) &num)))
-            throw Error("Network I/O error. Could not read the array length.\nThis may be due to a bug in libdap or a problem with\nthe network connection.");
+	um.get_int( (int &)num ) ;
 
         DBG(cerr << "Vector::deserialize: num = " << num << endl);
         DBG(cerr << "Vector::deserialize: length = " << length() << endl);
@@ -545,15 +591,9 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
         }
 
         if (_var->type() == dods_byte_c)
-            status = (0 != xdr_bytes(source, (char **) & _buf, &num,
-                                     DODS_MAX_ARRAY));
+	    um.get_vector( (char **)&_buf, num, *this ) ;
         else
-            status = (0 != xdr_array(source, (char **) & _buf, &num,
-                                     DODS_MAX_ARRAY, _var->width(),
-                                     (xdrproc_t)(_var->xdr_coder())));
-
-        if (!status)
-            throw Error("Network I/O error. Could not read packed array data.\nThis may be due to a bug in libdap or a problem with\nthe network connection.");
+	    um.get_vector( (char **)&_buf, num, _var->width(), *this ) ;
 
         DBG(cerr << "Vector::deserialize: read " << num << " elements\n");
 
@@ -561,8 +601,7 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
 
     case dods_str_c:
     case dods_url_c:
-        if ((0 == xdr_int(source, (int *) &num)))
-            throw Error("Network I/O error. Could not read the array length.\nThis may be due to a bug in libdap or a problem with\nthe network connection.");
+	um.get_int( (int &)num ) ;
 
         if (length() == -1)
             set_length(num);
@@ -575,8 +614,7 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
 
         for (i = 0; i < num; ++i) {
             string str;
-            if (!xdr_str(source, str))
-                throw Error("Network I/O Error. Could not read string data. This may be due to a\nbug in libdap or a problem with the network connection.");
+	    um.get_str( str ) ;
             d_str[i] = str;
 
         }
@@ -587,8 +625,7 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
     case dods_structure_c:
     case dods_sequence_c:
     case dods_grid_c:
-        if ((0 == xdr_int(source, (int *) &num)))
-            throw Error("Network I/O error. Could not read the array length.\nThis may be due to a bug in libdap or a problem with\nthe network connection.");
+	um.get_int( (int &)num ) ;
 
         if (length() == -1)
             set_length(num);
@@ -600,7 +637,7 @@ bool Vector::deserialize(XDR * source, DDS * dds, bool reuse)
 
         for (i = 0; i < num; ++i) {
             _vec[i] = _var->ptr_duplicate();
-            _vec[i]->deserialize(source, dds);
+            _vec[i]->deserialize(um, dds);
         }
 
         break;
@@ -815,6 +852,8 @@ void Vector::set_vec(unsigned int i, BaseType * val)
     _vec[i] = val->ptr_duplicate();
 }
 
+
+//@{
 /** @brief set the value of a byte array */
 bool
 Vector::set_value(dods_byte *val, int sz)
@@ -830,6 +869,24 @@ Vector::set_value(dods_byte *val, int sz)
     }
 }
 
+/** @brief set the value of a byte array */
+bool
+Vector::set_value(vector<dods_byte> &val, int sz)
+{
+    if (var()->type() == dods_byte_c) {
+	dods_byte *tmp_buf = new dods_byte[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a int16 array */
 bool
 Vector::set_value(dods_int16 *val, int sz)
@@ -845,6 +902,24 @@ Vector::set_value(dods_int16 *val, int sz)
     }
 }
 
+/** @brief set the value of a int16 array */
+bool
+Vector::set_value(vector<dods_int16> &val, int sz)
+{
+    if (var()->type() == dods_int16_c) {
+	dods_int16 *tmp_buf = new dods_int16[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a int32 array */
 bool
 Vector::set_value(dods_int32 *val, int sz)
@@ -860,6 +935,24 @@ Vector::set_value(dods_int32 *val, int sz)
     }
 }
 
+/** @brief set the value of a int32 array */
+bool
+Vector::set_value(vector<dods_int32> &val, int sz)
+{
+    if (var()->type() == dods_int32_c) {
+	dods_int32 *tmp_buf = new dods_int32[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a uint16 array */
 bool
 Vector::set_value(dods_uint16 *val, int sz)
@@ -875,6 +968,24 @@ Vector::set_value(dods_uint16 *val, int sz)
     }
 }
 
+/** @brief set the value of a uint16 array */
+bool
+Vector::set_value(vector<dods_uint16> &val, int sz)
+{
+    if (var()->type() == dods_uint16_c) {
+	dods_uint16 *tmp_buf = new dods_uint16[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a uint32 array */
 bool
 Vector::set_value(dods_uint32 *val, int sz)
@@ -890,6 +1001,24 @@ Vector::set_value(dods_uint32 *val, int sz)
     }
 }
 
+/** @brief set the value of a uint32 array */
+bool
+Vector::set_value(vector<dods_uint32> &val, int sz)
+{
+    if (var()->type() == dods_uint32_c) {
+	dods_uint32 *tmp_buf = new dods_uint32[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a float32 array */
 bool
 Vector::set_value(dods_float32 *val, int sz)
@@ -905,6 +1034,24 @@ Vector::set_value(dods_float32 *val, int sz)
     }
 }
 
+/** @brief set the value of a float32 array */
+bool
+Vector::set_value(vector<dods_float32> &val, int sz)
+{
+    if (var()->type() == dods_float32_c) {
+	dods_float32 *tmp_buf = new dods_float32[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a float64 array */
 bool
 Vector::set_value(dods_float64 *val, int sz)
@@ -923,6 +1070,24 @@ Vector::set_value(dods_float64 *val, int sz)
     }
 }
 
+/** @brief set the value of a float64 array */
+bool
+Vector::set_value(vector<dods_float64> &val, int sz)
+{
+    if (var()->type() == dods_float64_c) {
+	dods_float64 *tmp_buf = new dods_float64[sz] ;
+        _buf = reinterpret_cast<char*>(tmp_buf) ;
+        for (register int t = 0; t < sz; t++) {
+            tmp_buf[t] = val[t] ;
+        }
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
 /** @brief set the value of a string or url array */
 bool
 Vector::set_value(string *val, int sz)
@@ -941,7 +1106,33 @@ Vector::set_value(string *val, int sz)
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief set the value of a string or url array */
+bool
+Vector::set_value(vector<string> &val, int sz)
+{
+    if (var()->type() == dods_str_c || var()->type() == dods_url_c) {
+        d_str.resize(sz);
+        for (register int t = 0; t < sz; t++) {
+            d_str[t] = val[t] ;
+        }
+        set_length(sz) ;
+        set_read_p(true);
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+//@}
+
+//@{
+/** @brief Get a copy of the data held by this variable. 
+    Read data from this variable's internal storage and load it into the 
+    memory referenced by \c b. The argument \c b must point to enough memory
+    to hold length() Bytes.
+    
+    @param b A pointer to the memory to hold the data; must be at least
+    length() * sizeof(dods_byte) in size.*/
 void Vector::value(dods_byte *b) const
 {
     if (b && _var->type() == dods_byte_c) {
@@ -949,7 +1140,7 @@ void Vector::value(dods_byte *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_uint16 *b) const
 {
     if (b && _var->type() == dods_uint16_c) {
@@ -957,7 +1148,7 @@ void Vector::value(dods_uint16 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_int16 *b) const
 {
     if (b && _var->type() == dods_int16_c) {
@@ -965,7 +1156,7 @@ void Vector::value(dods_int16 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_uint32 *b) const
 {
     if (b && _var->type() == dods_uint32_c) {
@@ -973,7 +1164,7 @@ void Vector::value(dods_uint32 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_int32 *b) const
 {
     if (b && _var->type() == dods_int32_c) {
@@ -981,7 +1172,7 @@ void Vector::value(dods_int32 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_float32 *b) const
 {
     if (b && _var->type() == dods_float32_c) {
@@ -989,7 +1180,7 @@ void Vector::value(dods_float32 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(dods_float64 *b) const
 {
     if (b && _var->type() == dods_float64_c) {
@@ -997,13 +1188,25 @@ void Vector::value(dods_float64 *b) const
     }
 }
 
-/** @brief Get a copy of the pointer to the data held by this variable. */
+/** @brief Get a copy of the data held by this variable. */
 void Vector::value(vector<string> &b) const
 {
     if (_var->type() == dods_byte_c)
         b = d_str;
 }
 
+/** Allocated memory and copy data into the new buffer. Return the new 
+    buffer's pointer. The caller must delete the storage. */
+void *Vector::value()
+{
+    void *buffer = new char[width()];
+
+    memcpy(buffer, _buf, width());
+    
+    return buffer;
+}
+//@}
+
 /** @brief Add the BaseType pointer to this constructor type
     instance.
 
@@ -1019,7 +1222,11 @@ void Vector::value(vector<string> &b) const
 void Vector::add_var(BaseType * v, Part)
 {
     // Delete the current template variable
-    delete _var;
+    if( _var )
+    {
+	delete _var;
+	_var = 0 ;
+    }
 
     // if 'v' is null, just set _var to null and exit.
     if (!v) {
