@@ -35,16 +35,6 @@
 #ifndef _vector_h
 #define _vector_h 1
 
-
-#ifdef WIN32
-#include <rpc.h>
-#include <winsock2.h>
-#else
-#include <rpc/types.h>
-#include <netinet/in.h>
-#include <rpc/xdr.h>
-#endif
-
 #ifndef _basetype_h
 #include "BaseType.h"
 #endif
@@ -118,12 +108,13 @@ public:
 
     virtual void set_length(int l);
 
+    virtual void intern_data(const string &dataset, ConstraintEvaluator &eval,
+                             DDS &dds);
     virtual bool serialize(const string &dataset, ConstraintEvaluator &eval,
-                           DDS &dds, XDR *sink, bool ce_eval = true);
-    virtual bool deserialize(XDR *source, DDS *dds, bool reuse = false);
+                           DDS &dds, Marshaller &m, bool ce_eval = true);
+    virtual bool deserialize(UnMarshaller &um, DDS *dds, bool reuse = false);
 
     virtual unsigned int val2buf(void *val, bool reuse = false);
-
     virtual unsigned int buf2val(void **val);
 
     void set_vec(unsigned int i, BaseType *val);
@@ -131,13 +122,21 @@ public:
     void vec_resize(int l);
 
     virtual bool set_value(dods_byte *val, int sz);
+    virtual bool set_value(vector<dods_byte> &val, int sz);
     virtual bool set_value(dods_int16 *val, int sz);
+    virtual bool set_value(vector<dods_int16> &val, int sz);
     virtual bool set_value(dods_uint16 *val, int sz);
+    virtual bool set_value(vector<dods_uint16> &val, int sz);
     virtual bool set_value(dods_int32 *val, int sz);
+    virtual bool set_value(vector<dods_int32> &val, int sz);
     virtual bool set_value(dods_uint32 *val, int sz);
+    virtual bool set_value(vector<dods_uint32> &val, int sz);
     virtual bool set_value(dods_float32 *val, int sz);
+    virtual bool set_value(vector<dods_float32> &val, int sz);
     virtual bool set_value(dods_float64 *val, int sz);
+    virtual bool set_value(vector<dods_float64> &val, int sz);
     virtual bool set_value(string *val, int sz);
+    virtual bool set_value(vector<string> &val, int sz);
 
     virtual void value(dods_byte *b) const;
     virtual void value(dods_int16 *b) const;
@@ -148,6 +147,8 @@ public:
     virtual void value(dods_float64 *b) const;
     virtual void value(vector<string> &b) const;
 
+    virtual void *value();
+
     virtual BaseType *var(const string &name = "", bool exact_match = true,
                           btp_stack *s = 0);
     virtual BaseType *var(const string &name, btp_stack &s);
