@@ -38,7 +38,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: cgi_util.cc 16612 2007-06-04 22:08:57Z jimg $"
+    {"$Id: cgi_util.cc 17156 2007-09-21 17:44:27Z jimg $"
     };
 
 #include <stdio.h>
@@ -310,19 +310,25 @@ ErrMsgT(const string &Msgt)
     char TimStr[TimLen];
 
     if (time(&TimBin) == (time_t) - 1)
-        strcpy(TimStr, "time() error           ");
+        strncpy(TimStr, "time() error           ", TimLen-1);
     else {
-        strcpy(TimStr, ctime(&TimBin));
+        strncpy(TimStr, ctime(&TimBin), TimLen-1);
         TimStr[TimLen - 2] = '\0'; // overwrite the \n
     }
-
+    
+#if 0
+	// This was removed because writing these values out 'leaks' system information.
+	// Since we're not going to write out the script or host, I also removed the 
+	// calls to getenv(). jhrg 8/7/2007
     const char *host_or_addr = getenv("REMOTE_HOST") ? getenv("REMOTE_HOST") :
                                getenv("REMOTE_ADDR") ? getenv("REMOTE_ADDR") : "local (a non-CGI run)";
     const char *script = getenv("SCRIPT_NAME") ? getenv("SCRIPT_NAME") :
                          "OPeNDAP server";
 
     cerr << "[" << TimStr << "] CGI: " << script << " failed for "
-    << host_or_addr << ": " << Msgt << endl;
+         << host_or_addr << ": " << Msgt << endl;
+#endif
+    cerr << "[" << TimStr << "] DAP server error: " << Msgt << endl;
 }
 
 // Given a pathname, return just the filename component with any extension
@@ -418,15 +424,11 @@ rfc822_date(const time_t t)
     struct tm *stm = gmtime(&t);
     char d[256];
 
-    sprintf(d, "%s, %02d %s %4d %02d:%02d:%02d GMT", days[stm->tm_wday],
+    snprintf(d, 255, "%s, %02d %s %4d %02d:%02d:%02d GMT", days[stm->tm_wday],
             stm->tm_mday, months[stm->tm_mon],
-#if 0
-            // On Solaris 2.7 this tm_year is years since 1900. 3/17/2000
-            // jhrg
-            stm->tm_year < 100 ? 1900 + stm->tm_year : stm->tm_year,
-#endif
             1900 + stm->tm_year,
             stm->tm_hour, stm->tm_min, stm->tm_sec);
+    d[255] = '\0';
     return string(d);
 }
 
@@ -517,6 +519,59 @@ set_mime_text(FILE *out, ObjectType type, const string &ver,
     fprintf(out, CRLF) ;
 }
 
+/** Generate an HTTP 1.0 response header for a text document. This is used
+    when returning a serialized DAS or DDS object.
+
+    @param strm Write the MIME header to this stream.
+    @param type The type of this this response. Defaults to
+    application/octet-stream.
+    @param ver The version string; denotes the libdap implementation
+    version.
+    @param enc How is this response encoded? Can be plain or deflate or the
+    x_... versions of those. Default is x_plain.
+    @param last_modified The time to use for the Last-Modified header value.
+    Default is zero which means use the current time. */
+void
+set_mime_text(ostream &strm, ObjectType type, const string &ver,
+              EncodingType enc, const time_t last_modified)
+{
+    strm << "HTTP/1.0 200 OK" << CRLF ;
+    if (ver == "") {
+        strm << "XDODS-Server: " << DVR << CRLF ;
+        strm << "XOPeNDAP-Server: " << DVR << CRLF ;
+    }
+    else {
+        strm << "XDODS-Server: " << ver.c_str() << CRLF ;
+        strm << "XOPeNDAP-Server: " << ver.c_str() << CRLF ;
+    }
+    strm << "XDAP: " << DAP_PROTOCOL_VERSION << CRLF ;
+
+    const time_t t = time(0);
+    strm << "Date: " << rfc822_date(t).c_str() << CRLF ;
+
+    strm << "Last-Modified: " ;
+    if (last_modified > 0)
+        strm << rfc822_date(last_modified).c_str() << CRLF ;
+    else
+        strm << rfc822_date(t).c_str() << CRLF ;
+
+    if (type == dap4_ddx)
+        strm << "Content-Type: text/xml" << CRLF ;
+    else
+        strm << "Content-Type: text/plain" << CRLF ;
+
+    // Note that Content-Description is from RFC 2045 (MIME, pt 1), not 2616.
+    // jhrg 12/23/05
+    strm << "Content-Description: " << descrip[type] << CRLF ;
+    if (type == dods_error) // don't cache our error responses.
+        strm << "Cache-Control: no-cache" << CRLF ;
+    // Don't write a Content-Encoding header for x-plain since that breaks
+    // Netscape on NT. jhrg 3/23/97
+    if (enc != x_plain)
+        strm << "Content-Encoding: " << encoding[enc] << CRLF ;
+    strm << CRLF ;
+}
+
 /** Generate an HTTP 1.0 response header for a html document.
 
     @param out Write the MIME header to this FILE pointer.
@@ -563,6 +618,52 @@ set_mime_html(FILE *out, ObjectType type, const string &ver,
     fprintf(out, CRLF) ;
 }
 
+/** Generate an HTTP 1.0 response header for a html document.
+
+    @param strm Write the MIME header to this stream.
+    @param type The type of this this response.
+    @param ver The version string; denotes the libdap implementation
+    version.
+    @param enc How is this response encoded? Can be plain or deflate or the
+    x_... versions of those. Default is x_plain.
+    @param last_modified The time to use for the Last-Modified header value.
+    Default is zero which means use the current time. */
+void
+set_mime_html(ostream &strm, ObjectType type, const string &ver,
+              EncodingType enc, const time_t last_modified)
+{
+    strm << "HTTP/1.0 200 OK" << CRLF ;
+    if (ver == "") {
+        strm << "XDODS-Server: " << DVR << CRLF ;
+        strm << "XOPeNDAP-Server: " << DVR << CRLF ;
+    }
+    else {
+        strm << "XDODS-Server: " << ver.c_str() << CRLF ;
+        strm << "XOPeNDAP-Server: " << ver.c_str() << CRLF ;
+    }
+    strm << "XDAP: " << DAP_PROTOCOL_VERSION << CRLF ;
+
+    const time_t t = time(0);
+    strm << "Date: " << rfc822_date(t).c_str() << CRLF ;
+
+    strm << "Last-Modified: " ;
+    if (last_modified > 0)
+        strm << rfc822_date(last_modified).c_str() << CRLF ;
+    else
+        strm << rfc822_date(t).c_str() << CRLF ;
+
+    strm << "Content-type: text/html" << CRLF ;
+    // See note above about Content-Description header. jhrg 12/23/05
+    strm << "Content-Description: " << descrip[type] << CRLF ;
+    if (type == dods_error) // don't cache our error responses.
+        strm << "Cache-Control: no-cache" << CRLF ;
+    // Don't write a Content-Encoding header for x-plain since that breaks
+    // Netscape on NT. jhrg 3/23/97
+    if (enc != x_plain)
+        strm << "Content-Encoding: " << encoding[enc] << CRLF ;
+    strm << CRLF ;
+}
+
 /** Write an HTTP 1.0 response header for our binary response document (i.e.,
     the DataDDS object).
 
@@ -608,6 +709,51 @@ set_mime_binary(FILE *out, ObjectType type, const string &ver,
     fprintf(out, CRLF) ;
 }
 
+/** Write an HTTP 1.0 response header for our binary response document (i.e.,
+    the DataDDS object).
+
+    @param strm Write the MIME header to this stream.
+    @param type The type of this this response. Defaults to
+    application/octet-stream.
+    @param ver The version string; denotes the libdap implementation
+    version.
+    @param enc How is this response encoded? Can be plain or deflate or the
+    x_... versions of those. Default is x_plain.
+    @param last_modified The time to use for the Last-Modified header value.
+    Default is zero which means use the current time.
+ */
+void
+set_mime_binary(ostream &strm, ObjectType type, const string &ver,
+                EncodingType enc, const time_t last_modified)
+{
+    strm << "HTTP/1.0 200 OK" << CRLF ;
+    if (ver == "") {
+        strm << "XDODS-Server: " << DVR << CRLF ;
+        strm << "XOPeNDAP-Server: " << DVR << CRLF ;
+    }
+    else {
+        strm << "XDODS-Server: " << ver.c_str() << CRLF ;
+        strm << "XOPeNDAP-Server: " << ver.c_str() << CRLF ;
+    }
+    strm << "XDAP: " << DAP_PROTOCOL_VERSION << CRLF ;
+
+    const time_t t = time(0);
+    strm << "Date: " << rfc822_date(t).c_str() << CRLF ;
+
+    strm << "Last-Modified: " ;
+    if (last_modified > 0)
+        strm << rfc822_date(last_modified).c_str() << CRLF ;
+    else
+        strm << rfc822_date(t).c_str() << CRLF ;
+
+    strm << "Content-Type: application/octet-stream" << CRLF ;
+    strm << "Content-Description: " << descrip[type] << CRLF ;
+    if (enc != x_plain)
+        strm << "Content-Encoding: " << encoding[enc] << CRLF ;
+
+    strm << CRLF ;
+}
+
 
 /** Generate an HTTP 1.0 response header for an Error object.
     @param out Write the MIME header to this FILE pointer.
@@ -636,6 +782,33 @@ set_mime_error(FILE *out, int code, const string &reason,
     fprintf(out, CRLF) ;
 }
 
+/** Generate an HTTP 1.0 response header for an Error object.
+    @param strm Write the MIME header to this stream.
+    @param code HTTP 1.0 response code. Should be 400, ... 500, ...
+    @param reason Reason string of the HTTP 1.0 response header.
+    @param version The version string; denotes the DAP spec and implementation
+    version. */
+void
+set_mime_error(ostream &strm, int code, const string &reason,
+               const string &version)
+{
+    strm << "HTTP/1.0 " << code << " " << reason.c_str() << CRLF ;
+    if (version == "") {
+        strm << "XDODS-Server: " << DVR << CRLF ;
+        strm << "XOPeNDAP-Server: " << DVR << CRLF ;
+    }
+    else {
+        strm << "XDODS-Server: " << version.c_str() << CRLF ;
+        strm << "XOPeNDAP-Server: " << version.c_str() << CRLF ;
+    }
+    strm << "XDAP: " << DAP_PROTOCOL_VERSION << CRLF ;
+
+    const time_t t = time(0);
+    strm << "Date: " << rfc822_date(t).c_str() << CRLF ;
+    strm << "Cache-Control: no-cache" << CRLF ;
+    strm << CRLF ;
+}
+
 
 /** Use this function to create a response signaling that the target of a
     conditional get has not been modified relative to the condition given in
@@ -652,6 +825,21 @@ set_mime_not_modified(FILE *out)
     fprintf(out, CRLF) ;
 }
 
+/** Use this function to create a response signaling that the target of a
+    conditional get has not been modified relative to the condition given in
+    the request. This will have to be a date until the servers support ETags.
+
+    @brief Send a `Not Modified' response.
+    @param strm Write the response to this stream. */
+void
+set_mime_not_modified(ostream &strm)
+{
+    strm << "HTTP/1.0 304 NOT MODIFIED" << CRLF ;
+    const time_t t = time(0);
+    strm << "Date: " << rfc822_date(t).c_str() << CRLF ;
+    strm << CRLF ;
+}
+
 /** Look for the override file by taking the dataset name and
     appending `.ovr' to it. If such a file exists, then read it in and
     store the contents in <tt>doc</tt>. Note that the file contents
