@@ -29,14 +29,14 @@
 // Authors:
 //      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>
 
-// This is the source to `geturl'; a simple tool to exercise the Connect
+// This is the source to `getdap'; a simple tool to exercise the Connect
 // class. It can be used to get naked URLs as well as the DAP2 DAS and DDS
 // objects.  jhrg.
 
 #include "config.h"
 
 static char rcsid[] not_used =
-    { "$Id: getdap.cc 16088 2007-03-28 21:42:19Z jimg $"
+    { "$Id: getdap.cc 17299 2007-10-24 19:39:50Z jimg $"
     };
 
 #include <stdio.h>
@@ -55,7 +55,7 @@ static char rcsid[] not_used =
 using std::cerr;
 using std::endl;
 
-const char *version = "$Revision: 16088 $";
+const char *version = CVER " (" DVR " DAP/" DAP_PROTOCOL_VERSION ")";
 
 extern int dods_keep_temps;     // defined in HTTPResponse.h
 
@@ -115,7 +115,7 @@ void usage(string name)
 bool read_data(FILE * fp)
 {
     if (!fp) {
-        fprintf(stderr, "geturl: Whoa!!! Null stream pointer.\n");
+        fprintf(stderr, "getdap: Whoa!!! Null stream pointer.\n");
         return false;
     }
     // Changed from a loop that used getc() to one that uses fread(). getc()
@@ -146,7 +146,7 @@ static void print_data(DDS & dds, bool print_rows = false)
 
 int main(int argc, char *argv[])
 {
-    GetOpt getopt(argc, argv, "idaDxXAVvkB:c:m:zsh?");
+    GetOpt getopt(argc, argv, "idaDxXAVvkB:c:m:zsh?H");
     int option_char;
 
     bool get_das = false;
@@ -190,7 +190,7 @@ int main(int argc, char *argv[])
             use_ais = true;
             break;
         case 'V':
-            fprintf(stderr, "geturl version: %s\n", version);
+            fprintf(stderr, "getdap version: %s\n", version);
             exit(0);
         case 'i':
             get_version = true;
@@ -235,7 +235,7 @@ int main(int argc, char *argv[])
                 fprintf(stderr, "Fetching: %s\n", argv[i]);
 
             string name = argv[i];
-            Connect *url = 0;;
+            Connect *url = 0;
             if (use_ais) {
                 if (!ais_db.empty())
                     url = new AISConnect(name, ais_db);
@@ -270,7 +270,7 @@ int main(int argc, char *argv[])
                         url->read_data(dds, r);
                     }
                     else {
-                        r = new Response(fopen(argv[i], "r"));
+                        r = new Response(fopen(argv[i], "r"), 0);
 
                         if (!r->get_stream())
                             throw Error(string("The input source: ")
@@ -375,7 +375,7 @@ int main(int argc, char *argv[])
                         fprintf(stderr, "DDX:\n");
                     }
 
-                    dds.print_xml(stdout, false, "geturl; no blob yet");
+                    dds.print_xml(stdout, false, "getdap; no blob yet");
                 }
             }
 
@@ -402,7 +402,7 @@ int main(int argc, char *argv[])
                         fprintf(stderr, "Client-built DDX:\n");
                     }
 
-                    dds.print_xml(stdout, false, "geturl; no blob yet");
+                    dds.print_xml(stdout, false, "getdap; no blob yet");
                 }
             }
 
