@@ -36,7 +36,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: util.cc 16731 2007-06-22 18:56:39Z jimg $"
+    {"$Id: util.cc 17506 2007-11-28 21:20:52Z jimg $"
     };
 
 #include <stdio.h>
@@ -73,6 +73,7 @@ static char rcsid[] not_used =
 #include "Error.h"
 #include "parser.h"
 #include "util.h"
+#include "GNURegex.h"
 #include "debug.h"
 
 
@@ -161,96 +162,18 @@ unique_names(vector<BaseType *> l, const string &var_name,
     return true;
 }
 
-// This function is used to allocate memory for, and initialize, a new XDR
-// pointer. It sets the stream associated with the (XDR *) to STREAM.
-//
-// NB: STREAM is not one of the C++/libg++ iostream classes; it is a (FILE
-// *).
-
-//  These func's moved to xdrutil_ppc.* under the PPC as explained there
-#ifndef __POWERPC__
-XDR *
-new_xdrstdio(FILE *stream, enum xdr_op xop)
-{
-    XDR *xdr = new XDR;
-
-    xdrstdio_create(xdr, stream, xop);
-
-    return xdr;
-}
-
-XDR *
-set_xdrstdio(XDR *xdr, FILE *stream, enum xdr_op xop)
-{
-    xdrstdio_create(xdr, stream, xop);
-
-    return xdr;
-}
-
-// Delete an XDR pointer allocated using the above function. Do not close the
-// associated FILE pointer.
-
-void
-delete_xdrstdio(XDR *xdr)
-{
-    xdr_destroy(xdr);
-
-    delete xdr; xdr = 0;
-}
-#endif
-
-// This function is used to en/decode Str and Url type variables. It is
-// defined as extern C since it is passed via function pointers to routines
-// in the xdr library where it is executed. This function is defined so
-// that Str and Url have an en/decoder which takes exactly two argumnets: an
-// XDR * and a string reference.
-//
-// NB: this function is *not* used for arrays (i.e., it is not the function
-// referenced by BaseType's _xdr_coder field when the object is a Str or Url.
-// Also note that \e max_str_len is an obese number but that really does not
-// matter; xdr_string() would never actually allocate that much memory unless
-// a string that size was sent from the server.
-//
-// Returns: XDR's bool_t; TRUE if no errors are detected, FALSE
-// otherwise. The formal parameter BUF is modified as a side effect.
-
-extern "C" bool_t
-xdr_str(XDR *xdrs, string &buf)
-{
-    DBG(cerr << "In xdr_str, xdrs: " << xdrs << endl);
-
-    switch (xdrs->x_op) {
-    case XDR_ENCODE: { // BUF is a pointer to a (string *)
-            const char *out_tmp = buf.c_str();
-
-            return xdr_string(xdrs, (char **)&out_tmp, max_str_len);
-        }
-
-    case XDR_DECODE: {
-            char *in_tmp = NULL;
-
-            bool_t stat = xdr_string(xdrs, &in_tmp, max_str_len);
-            if (!stat)
-                return stat;
-
-            buf = in_tmp;
-
-            free(in_tmp);
-
-            return stat;
-        }
-
-    default:
-        assert(false);
-        return 0;
-    }
-}
-
 const char *
 libdap_root()
 {
+    return LIBDAP_ROOT;
+#if 0
+    // I've changed this because this could be used to get the library to
+    // use a different compression function when it builds compressed 
+    // responses. The use of 'deflate' to compress responses should be
+    // removed since Hyrax now uses Tomcat to perform this function.
     char *libdap_root = 0;
     return ((libdap_root = getenv("LIBDAP_ROOT")) ? libdap_root : LIBDAP_ROOT);
+#endif
 }
 
 extern "C"
@@ -571,28 +494,55 @@ path_to_filename(string path)
 //
 // This function allocates storage using new. The caller must delete the char
 // array.
+
+// Change this to a version that either returns a string or an open file
+// descriptor. Use information from https://buildsecurityin.us-cert.gov/
+// (see open()) to make it more secure. Ideal solution: get deserialize()
+// methods to read from a stream returned by libcurl, not from a temporary
+// file. 9/21/07 jhrg
 char *
 get_tempfile_template(char *file_template)
 {
     char *c;
+    
 #ifdef WIN32
-    if (getenv("TEMP") && (access(getenv("TEMP"), 6) == 0))
-        c = getenv("TEMP");
-    else if (getenv("TMP"))
-        c = getenv("TMP");
-#else
-    if (getenv("TMPDIR") && (access(getenv("TMPDIR"), W_OK | R_OK) == 0))
-        c = getenv("TMPDIR");
-#ifdef P_tmpdir
-    else if (access(P_tmpdir, W_OK | R_OK) == 0)
-        c = P_tmpdir;
-#endif
-#endif
-    else
-        c = ".";
+    // whitelist for a WIN32 directory
+    Regex directory("[-a-zA-Z0-9_\\]*");
+	
+    c = getenv("TEMP");
+    if (c && directory.match(c, strlen(c)) && (access(getenv("TEMP"), 6) == 0)
+    	goto valid_temp_directory;
 
-    char *temp = new char[strlen(c) + strlen(file_template) + 2];
-    strcpy(temp, c);
+    c= getenv("TMP");
+    if (c && directory.match(c, strlen(c)) && (access(getenv("TEMP"), 6) == 0)
+    	goto valid_temp_directory;
+#else
+	// whitelist for a directory
+	Regex directory("[-a-zA-Z0-9_/]*");
+	
+	c = getenv("TMPDIR");
+	if (c && directory.match(c, strlen(c)) && (access(c, W_OK | R_OK) == 0))
+    	goto valid_temp_directory;
+
+#ifdef P_tmpdir
+	if (access(P_tmpdir, W_OK | R_OK) == 0) {
+        c = P_tmpdir;
+        goto valid_temp_directory;
+	}
+#endif
+
+#endif  // WIN32
+
+    c = ".";
+    
+valid_temp_directory:
+	// Sanitize allocation
+	int size = strlen(c) + strlen(file_template) + 2;
+	if (!size_ok(1, size))
+		throw Error("Bad temporary file name.");
+		
+    char *temp = new char[size];
+    strncpy(temp, c, size-2);
     strcat(temp, "/");
 
     strcat(temp, file_template);
@@ -631,3 +581,55 @@ file_to_string(FILE *fp)
         oss << c;
     return oss.str();
 }
+
+/** @name Security functions */
+//@{
+
+/** @brief sanitize the size of an array.
+    Test for integer overflow when dynamically allocating an array.
+    @param nelem Number of elements.
+    @param sz size of each element.
+    @return True if the \c nelem elements of \c sz size will overflow an array. */
+bool
+size_ok(uint sz, uint nelem)
+{
+    return (sz > 0 && nelem < UINT_MAX / sz);
+}
+
+/** @brief Does the string name a potentailly valid pathname?
+    Test the given pathname to verfiy that it is a valid name. We define this 
+    as: Contains only printable characters; and Is less then 256 characters.
+    If \e strict is true, test that the pathname consists of only letters, 
+    digits, and underscore, dash and dot characters instead of the more general
+    case where a pathname can be composed of any printable characters.
+    
+    @note Using this function does not guarentee that the path is valid, only
+    that the path \e could be valid. The intent is foil attacks where an
+    exploit is encoded in a string then passed to a library function. This code
+    does not address whether the pathname references a valid resource.
+    
+    @param path The pathname to test
+    @param strict Apply more restrictive tests (true by default) 
+    @return true if the pathname consists of legal characters and is of legal
+    size, false otherwise. */
+bool
+pathname_ok(const string &path, bool strict)
+{
+    if (path.length() > 255)
+        return false;
+    
+    Regex name("[0-9A-z_./-]+");
+    if (!strict)
+        name = "[:print:]+";
+        
+    string::size_type len = path.length();
+    int result = name.match(path.c_str(), len);
+    // Protect against casting too big an uint to int
+    // if LEN is bigger than the max int32, the second test can't work
+    if (len > INT_MAX || result != static_cast<int>(len))
+        return false;
+ 
+    return true;
+}
+
+//@}
